Package: DSS
Title: Dispersion shrinkage for sequencing data
Version: 2.60.0
Date: 2026-4-9
Author: Hao Wu<hao.wu@emory.edu>, Hao Feng<hxf155@case.edu>
Depends: R (>= 3.5.0), methods, Biobase, BiocParallel, bsseq, parallel
Imports: utils, graphics, stats, splines
Maintainer: Hao Wu <hao.wu@emory.edu>, Hao Feng <hxf155@case.edu>
Description: DSS is an R library performing differntial analysis for
        count-based sequencing data. It detectes differentially
        expressed genes (DEGs) from RNA-seq, and differentially
        methylated loci or regions (DML/DMRs) from bisulfite sequencing
        (BS-seq). The core of DSS is a new dispersion shrinkage method
        for estimating the dispersion parameter from Gamma-Poisson or
        Beta-Binomial distributions.
License: GPL
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, rmarkdown, edgeR
biocViews: Sequencing, RNASeq, DNAMethylation,GeneExpression,
        DifferentialExpression,DifferentialMethylation
Config/pak/sysreqs: make libbz2-dev liblzma-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:36:36 UTC
RemoteUrl: https://github.com/bioc/DSS
RemoteRef: RELEASE_3_23
RemoteSha: a954918494c51ec343aa885105c0b80ec69e4631
NeedsCompilation: yes
Packaged: 2026-04-29 13:50:48 UTC; root
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 13:55:50 UTC; unix

useDynLib(DSS)

import(methods)
import(utils)
import(graphics)
import(stats)
importFrom(splines, ns)
import(Biobase)
import(BiocParallel)
import(bsseq)

exportClasses("SeqCountSet")
exportMethods(
              normalizationFactor, "normalizationFactor<-",
              dispersion, "dispersion<-"
              )

export(
       newSeqCountSet,
       estDispersion,
       estNormFactors,
       waldTest,
       DSS.DE,
       DMLtest,
       callDML,
       callDMR,
       makeBSseqData,
       showOneDMR,
       DMLfit.multiFactor,
       DMLtest.multiFactor
)

###############################################
### some utility functions
###############################################

### generate negative binomial rv, given mu and phi (over-dispersion)
rnegbinom <- function (n, mu =1, phi=0.01){
    rpois(n, rgamma(n, shape=1/phi,scale=mu*phi))
}


### Smoothing function. Smooth by chr
smooth.chr <- function(x, ws, allchr, allpos, method=c("avg", "sum")) {
    method <- match.arg(method)
    if(method == "avg")
        flag = 1
    else
      flag = 0

    ## remove NA's
    ix = !is.na(x)
    x2 = x[ix]
    allchr2 = allchr[ix]
    allpos2 = allpos[ix]

    n0=length(x2)
    idx=split(1:n0, allchr2)
    res2=rep(0, n0)
    for(i in seq(along=idx)) {
        res2[idx[[i]]]=.Call("windowFilter", x2[idx[[i]]], as.integer(allpos2[idx[[i]]]), as.integer(ws), as.integer(flag))
    }

    if(sum(ix) > 0) {
        res = rep(NA, length(x))
        res[ix] = res2
    } else res = res2

    return(res)
}

######################################################################
##
## a list of functions for testing DML from Bisulfite seq data
##
######################################################################

######################################
## wrapper function for DML test
######################################
DMLtest <- function(BSobj, group1, group2, equal.disp=FALSE, smoothing=FALSE,
                    smoothing.span=500, ncores) {

    ## determine number of cores used in parallel computing setting
    if(missing(ncores)) {
        if(.Platform$OS.type == "windows" | Sys.info()['sysname'] == "Windows")
            ## Windows, use single core
            ncores = 1
        else # for Mac and linux
            ncores = max(detectCores() - 3, 1)
    }

    if(ncores > detectCores())
        stop("ncores exceeds the number of CPU cores on the system.")

    ## grab two group data
    tmp <- getBSseqIndex(sampleNames(BSobj), group1, group2)
    BS1 <- BSobj[,tmp$group1]
    BS2 <- BSobj[,tmp$group2]

    ## remove loci with all 0 coverages in a condition
    ## It's not required that all replicates have coverage.
    ## But there must be some coverage from at least one replicate.
    n1 <- as.array(getBSseq(BS1, "Cov"))
    n2 <- as.array(getBSseq(BS2, "Cov"))
    ## remove if a rather long strech (like 200 bps) of regions have no coverage
    allpos <- start(BSobj)
    ix1 <- hasCoverage(n1, allpos)
    ix2 <- hasCoverage(n2, allpos)
    ix <- ix1 & ix2
    BS1 <- BS1[ix]
    BS2 <- BS2[ix]

    ## Check the consistence of inputs
    nreps1 <- dim(BS1)[2]
    nreps2<- dim(BS2)[2]
    if( (nreps1==1 | nreps2==1) & !equal.disp ) { ## singel replicate case, and unequal dispersion in two groups
        if( !smoothing )
            stop("There is no biological replicates in at least one condition. Please set smoothing=TRUE or equal.disp=TRUE and retry.")
    }

    if(!smoothing) { ## no smoothing.
        dmls <- DMLtest.noSmooth(BS1, BS2, equal.disp, ncores)
    } else { ## smoothing version
        dmls <- DMLtest.Smooth(BS1, BS2, equal.disp, smoothing.span, ncores)
    }

    class(dmls)[2] = "DMLtest"
    invisible(dmls)
}

##################################################
## determine sample index from a BSseq object
##################################################
getBSseqIndex <- function(sName, group1, group2) {
    check <- function(group, id) {
        thisGrp = paste("group", id, sep="")

        if (is.character(group)) {
            if(!all(group %in% sName))
                stop("Some sample names not found in", thisGrp)
            group <- match(group, sName)
        }
        if (is.numeric(group)) {
            if(min(group) < 1 | max(group) > length(sName))
                stop("Some group indices are wrong in", thisGrp)
        }
        else stop(paste("problems with argument", thisGrp))
        group
    }

    group1 <- check(group1, 1)
    group2 <- check(group2, 2)

    if(length(intersect(group1, group2)) > 0)
        stop("Two groups have common sample.")

    if(length(group1) <= 0)
        stop("group1 not found.")
    if(length(group2) <= 0)
        stop("group2 not found.")

    list(group1=group1, group2=group2)
}


######################################
## test DML without smoothing
######################################
DMLtest.noSmooth <- function(BS1, BS2, equal.disp, ncores) {
    ## grab counts
    x1 <- as.array(getCoverage(BS1, type="M"))
    n1 <- as.array(getCoverage(BS1, type="Cov"))
    x2 <- as.array(getCoverage(BS2, type="M"))
    n2 <- as.array(getCoverage(BS2, type="Cov"))
    nreps1 <- ncol(x1)
    nreps2 <- ncol(x2)

    ## estimate means
    estprob1 <- compute.mean.noSmooth(x1, n1)
    estprob2 <- compute.mean.noSmooth(x2, n2)

    ## estimate dispersion
    ## - this part is slow. Could be computed parallely. Will implement later.
    cat("Estimating dispersion for each CpG site, this will take a while ...\n")
    if(equal.disp | nreps1==1 | nreps2==1) {
        phi1 <- phi2 <- est.dispersion.BSseq(cbind(x1,x2), cbind(n1,n2),
                                             cbind(estprob1, estprob2), ncores)
    } else {
        phi1 <- est.dispersion.BSseq(x1, n1, estprob1, ncores)
        phi2 <- est.dispersion.BSseq(x2, n2, estprob2, ncores)
    }


    ## weight the counts
    wt1 <- 1 / (1+(n1-1)*phi1);    wt1 <- wt1 / mean(wt1)
    wt2 <- 1 / (1+(n2-1)*phi2);    wt2 <- wt2 / mean(wt2)
    x1.wt <- x1*wt1
    n1.wt <- n1*wt1
    x2.wt <- x2*wt2
    n2.wt <- n2*wt2

    ## re-estimate means
    estprob1 <- compute.mean.noSmooth(x1.wt, n1.wt)
    estprob2 <- compute.mean.noSmooth(x2.wt, n2.wt)

    ## perform Wald test
    allchr <- as.character(seqnames(BS1))
    allpos <- start(BS1)
    wald <- waldTest.DML(x1.wt, n1.wt, estprob1, phi1, x2.wt, n2.wt, estprob2, phi2,
                         smoothing=FALSE, allchr=allchr, allpos=allpos)

    return(wald)

}

######################################
## test DML with smoothing
######################################
DMLtest.Smooth <- function(BS1, BS2, equal.disp, smoothing.span, ncores) {
    ## grab counts
    x1 <- as.array(getCoverage(BS1, type="M"))
    n1 <- as.array(getCoverage(BS1, type="Cov"))
    x2 <- as.array(getCoverage(BS2, type="M"))
    n2 <- as.array(getCoverage(BS2, type="Cov"))
    nreps1 <- ncol(x1)
    nreps2 <- ncol(x2)
    allchr <- as.character(seqnames(BS1))
    allpos <- start(BS1)

    ## Smoothing
    cat("Smoothing ...\n")
    estprob1 <- compute.mean.Smooth(x1, n1, allchr, allpos, smoothing.span)
    estprob2 <- compute.mean.Smooth(x2, n2, allchr, allpos, smoothing.span)

    ## estimate priors from counts
    cat("Estimating dispersion for each CpG site, this will take a while ...\n")
    if(equal.disp) {
        phi1 <- phi2 <- est.dispersion.BSseq(cbind(x1,x2), cbind(n1,n2), cbind(estprob1, estprob2), ncores)
    } else {
        phi1 <- est.dispersion.BSseq(x1, n1, estprob1, ncores)
        phi2 <- est.dispersion.BSseq(x2, n2, estprob2, ncores)
    }

    ## update counts - weight by dispersion
    wt1 <- 1 / (1+(n1-1)*phi1); wt1 <- wt1 / mean(wt1, na.rm=TRUE)
    wt2 <- 1 / (1+(n2-1)*phi2); wt2 <- wt2 / mean(wt2, na.rm=TRUE)
    x1.wt <- x1*wt1
    n1.wt <- n1*wt1
    x2.wt <- x2*wt2
    n2.wt <- n2*wt2

    ## re-estimate means
    estprob1 <- compute.mean.Smooth(x1.wt, n1.wt, allchr, allpos, smoothing.span)
    estprob2 <- compute.mean.Smooth(x2.wt, n2.wt, allchr, allpos, smoothing.span)

    cat("Computing test statistics ...\n")
    wald <- waldTest.DML(x1.wt, n1.wt, estprob1, phi1, x2.wt, n2.wt, estprob2, phi2,
                         smoothing=TRUE, smoothing.span, allchr=allchr, allpos=allpos)
##     wald <- waldTest.DML(x1, n1, estprob1, phi1, x2, n2, estprob2, phi2,
##                          smoothing=TRUE, smoothing.span, allchr=allchr, allpos=allpos)

    return(wald)
}


###############################################################################
## Perform Wald tests for calling DML
###############################################################################
waldTest.DML <- function(x1,n1,estprob1, phi1, x2,n2, estprob2, phi2, smoothing,
                         smoothing.span, allchr, allpos) {

    ## Wald test
    if(smoothing) {
        wald <- compute.waldStat.Smooth(estprob1[,1], estprob2[,1], n1, n2, phi1, phi2,
                                        smoothing.span, allchr, allpos)
    } else {
        wald <- compute.waldStat.noSmooth(estprob1[,1], estprob2[,1], n1, n2, phi1, phi2)
    }

    ## combine with chr/pos and output
    result <- data.frame(chr=allchr, pos=allpos, wald)
    ## remove NA entries - Maybe I should keep them so result have the same dimension as inputs???
    ii <- !is.na(result$stat)
    result <- result[ii,]

    ## sort result according to chr and pos - maybe this is not important??
    ix <- sortPos(result$chr, result$pos)
    result <- result[ix,]
    return(result)
}

###########################################################
## compute Wald test statistics when there's no smoothing
###########################################################
compute.waldStat.noSmooth <- function(estprob1, estprob2, n1, n2, phi1, phi2) {
    ##rowSums <- DelayedArray::rowSums
    dif <- estprob1 - estprob2
    n1m <- rowSums(n1);    n2m <- rowSums(n2)
    var1 <- rowSums(n1*estprob1*(1-estprob1)*(1+(n1-1)*phi1)) / (n1m)^2
    var2 <- rowSums(n2*estprob2*(1-estprob2)*(1+(n2-1)*phi2)) / (n2m)^2
    ##vv <- var1/ncol1+var2/ncol2
    vv <- var1 + var2
    ## bound vv a little bit??
    vv[vv<1e-5] <- 1e-5
    se <- sqrt(vv)
    stat <- dif/se
    pval <- 2 * pnorm(-abs(stat)) ## p-value for hypothesis testing
    fdr <- p.adjust(pval, method="fdr")

    data.frame(mu1=estprob1, mu2=estprob2, diff=dif, diff.se=se, stat=stat,
               phi1=phi1, phi2=phi2, pval=pval, fdr=fdr)
}

###########################################################
## compute Wald test statistics when there is smoothing
## Note that the variance computation is different when there's smoothing!!!
## The variances will be smaller in the CG dense regions.
## This is reasonable because there're more data points in smoothing.
## But does it make sense biologically???
###########################################################
compute.waldStat.Smooth <- function(estprob1, estprob2, n1, n2, phi1, phi2, smoothing.span,
                                    allchr, allpos) {
    dif <- estprob1 - estprob2
    ## compute variances for moving average values. This is tricky!
    var1 <- compute.var.Smooth(estprob1, n1, phi1, smoothing.span, allchr, allpos)
    var2 <- compute.var.Smooth(estprob2, n2, phi2, smoothing.span, allchr, allpos)

    ##  var1 <- compute.var.Smooth.old(estprob1, n1, phi1, smoothing.span, allchr, allpos)
    ##  var2 <- compute.var.Smooth.old(estprob2, n2, phi2, smoothing.span, allchr, allpos)

    vv <- var1 + var2
    ## bound vv a little bit
    vv[vv<1e-5] <- 1e-4
    vv[vv> 1- 1e-5] <- 1-1e-4

    se <- sqrt(vv)
    stat <- dif/se
    pval <- 2 * pnorm(-abs(stat)) ## p-value for hypothesis testing
    fdr <- p.adjust(pval, method="fdr")

    data.frame(mu1=estprob1, mu2=estprob2, diff=dif, diff.se=se, stat=stat,
               phi1=phi1, phi2=phi2, pval=pval, fdr=fdr)
}


################################################################
## function to compute the variance for moving average values
################################################################
compute.var.Smooth <- function(estprob1, n1, phi1, smoothing.span, allchr, allpos) {
    nreps <- ncol(n1)
    ## estimate distance-dependent autocorrelations
    ## rhos <- est.rho(estprob1, allchr, allpos)
    ## autocorrelation.
    rho <- 0.8
    ## rho <- acf(estprob1, 1, plot=FALSE)$acf[2]  ## lag-1 autocorrelation, use 0.8 fixed
    ## compute vars for each replicate
    vars.rep <- matrix(0, nrow=nrow(n1), ncol=nreps)
    idx <- split(1:length(allchr), allchr)

    ## note the smoothed value were calculated with a small constant added to N and X.
    ## Need to account for that!!!
    const <- 1/ncol(n1)
    n1 <- n1 + const

    for(irep in 1:nreps) { ## loop on replicates
        ## variances at each position
        vars <- n1[,irep] * estprob1*(1-estprob1) * (1+(n1[,irep]-1)*phi1)
        ## 'vars' could be a DelayedArray object so turn it into an ordinary
        ## array
        vars <- as.array(vars)
        ## compute covariances - do by chr
        tmp1 <- estprob1*(1-estprob1)*phi1
        vars.smooth <- rep(0, length(allchr))
        for(i in seq(along=idx)) { ## loop on chromosomes
            thisidx <- idx[[i]]
            vars.smooth[idx[[i]]]=.Call( "compute_var_smooth", vars[thisidx], tmp1[thisidx],
                       as.double(n1[thisidx,irep]), as.integer(allpos[thisidx]),
                       as.integer(smoothing.span), as.double(rho) )
        }
        vars.rep[,irep] <- vars.smooth
    }
    ## compute denominators
    flag <- "sum"
    n1.sm <- as.array(n1)
    for(i in 1:nreps)
        n1.sm[,i] <- smooth.chr(as.double(n1[,i]), smoothing.span, allchr, allpos, flag)
    denom <- rowSums(n1.sm) ^ 2
    ## results
    vars <- rowSums(vars.rep) / denom
    vars
}



################################################################
## old function to compute the variance for moving average values.
## This uses an approximation that will speed up the calculation.
################################################################
compute.var.Smooth.old <- function(estprob1, n1, phi1, smoothing.span, allchr, allpos) {
    ## variance of X at each position for each replicate
    var1.X <- (n1*estprob1*(1-estprob1)*(1+(n1-1)*phi1))

    ## Consider the smoothing effect
    n1.sm <- n1
    var1.X.sm <- var1.X
    nCG1 <- n1

    flag <- "sum"
    for(i in 1:ncol(n1)) {
        n1.sm[,i] <- smooth.chr(as.double(n1[,i]), smoothing.span, allchr, allpos, flag)
        var1.X.sm[,i] <- smooth.chr(as.double(var1.X[,i]), smoothing.span, allchr, allpos, flag)
        nCG1[,i] <- smooth.chr(rep(1.0, nrow(n1)), smoothing.span, allchr, allpos, flag)
    }
    ## adjust for correlation - this is very crude, but serve the purpose
    ## Need to work this out !!!!
    var1.X.sm2 <- var1.X.sm #* nCG1 * 0.1
    n1m <- rowSums(n1.sm)
    var1 <- rowSums(var1.X.sm2) / n1m^2
    var1
}

################################################
## wrapper function for calling DML
################################################
callDML <- function(DMLresult, delta=0.1, p.threshold=1e-5) {

    if(class(DMLresult)[2] == "DMLtest.multiFactor" & delta>0) {
        warning("'delta' cannot be specified for multiple-factor test results. Set delta=0 and proceed ...\n")
        delta = 0
    }

    ## obtain posterior probability that the differnce of two means are greater than a threshold
    if( delta > 0 ) {
        p1 <- pnorm(DMLresult$diff-delta, sd=DMLresult$diff.se) ## Pr(delta.mu > delta)
        p2 <- pnorm(DMLresult$diff+delta, sd=DMLresult$diff.se, lower.tail=FALSE) ## Pr(-delta.mu < -delta)
        postprob.overThreshold <- p1 + p2
        DMLresult <- data.frame(DMLresult, postprob.overThreshold=postprob.overThreshold)
        scores <- 1 - postprob.overThreshold
    } else {
        scores <- DMLresult$pval
    }

    ix <- scores < p.threshold
    result <- DMLresult[ix,]

    ## sort by score
    ii <- sort(scores[ix], index.return=TRUE)$ix
    result[ii,]

}


####################################################################
## function to determine what loci to keep, based on coverage depth
####################################################################
hasCoverage <- function(nn, allpos, thresh=2) {
    nn2 <- rowSums(nn)
    ws <- 200
    flag <- 0
    nn.sm <- .Call("windowFilter", as.double(nn2), as.integer(allpos), as.integer(ws), as.integer(flag))
    nn.sm > thresh*ncol(nn)
}

############################################################################
## Wrapper function for Fitting multiple factor DM model.
## Take a bsseq object and a design matrix
############################################################################
DMLfit.multiFactor <- function(BSobj, design, formula, smoothing=FALSE, smoothing.span=500) {
    ## some checking to make sure input data is correct
    if(length(sampleNames(BSobj)) != nrow(design))
        stop("Dimension of data and design don't match. ")

    ## make design matrix out of formula
    X <- model.matrix(formula, design)
    if(nrow(X) <= ncol(X)+1)
        stop("No enough degree of freedom to fit the linear model. Drop some terms in formula.")

    ## take counts from BSobj
    N0 <- getBSseq(BSobj, "Cov")
    Y0 <- getBSseq(BSobj, "M")

    ## compute the response variable Z, which is transformed methylation level
    c0 = 0.1
    if(smoothing) { ## with smoothing. The mean methylation levels are smoothed
        allchr <- as.character(seqnames(BSobj))
        allpos <- start(BSobj)
        N0.sm = N0; Y0.sm = Y0
        for(i in 1:ncol(N0)) {
            N0.sm[,i] <- round(smooth.chr(as.double(N0[,i]), smoothing.span, allchr, allpos, "avg"))
            Y0.sm[,i] <- round(smooth.chr(as.double(Y0[,i]), smoothing.span, allchr, allpos, "avg"))
        }
        Z0 = asin(2*(Y0.sm+c0)/(N0.sm+2*c0) - 1)

    } else { ## no smoothing
        Z0 = asin(2*(Y0+c0)/(N0+2*c0) - 1)
    }

    ## fit the model
    fit <- DMLfit.multiFactor.engine(as.array(Y0), as.array(N0), X, as.array(Z0))

    ## return
    list(gr=getBSseq(BSobj, "gr"), design=design, formula=formula, X=X, fit=fit)

}


############################################################################
## Engine function for Fitting multiple factor DM model.
############################################################################
DMLfit.multiFactor.engine <- function(Y0, N0, X0, Z0) {

    if( (!is.matrix(Y0) | !is.matrix(N0)) )
        stop("Y and N need to be matrices.\n")

    ## get dimensions
    p = NCOL(X0)
    n = NROW(X0)
    C = nrow(Y0)

    ## loop over CpG sites
    beta = matrix(NA, nrow=C, ncol=p)
    var.beta = matrix(NA, nrow=C, ncol=p*p)
    phi = numeric(C)

    cat("Fitting DML model for CpG site: ")
    for( i in 1:C ) {
        if(i %% 1e5 == 0)
            cat(i, ", ")
        ## take counts for current CpG and fit model
        tmp = DMLfit.oneCG(Y0[i,], N0[i,], X0, Z0[i,], n, p)
        if(is.null(tmp)) next
        ## save point estimates and SE for this CpG
        beta[i,] = tmp$beta0
        var.beta[i,] = tmp$var.beta0
        phi[i] = tmp$phi
    }

    list(beta=beta, var.beta=var.beta, phi=phi)
}


##############################################################
## DML model fitting for one CpG
## This is the "core" function and all methods are in here!!
##############################################################

DMLfit.oneCG <- function(Y, N, X, Z, n, p) {
    ## small constants to bound p and phi
    c1 = 0.001

    ## check to make sure data is complete
    ix <- N > 0
    if(mean(ix) < 1) { ## has missing entries
        X <- X[ix,,drop=FALSE]
        Y <- Y[ix]
        N <- N[ix]
        ## check design
        if(nrow(X) < ncol(X) + 1) ## not enough df for regression
            return(NULL)
        if(any(abs(svd(X)$d) <1e-8)) ## design is not of full rank because of missing. Skip
            return(NULL)
        Z <- Z[ix]
    }

    ## Transform the methylation levels. Add a small constant to bound away from 0/1.
    # Z = asin(2*(Y+c0)/(N+2*c0) - 1)

    ## First round of weighted least square.
    ## QR decomposition has to be performed for each CpG because it's WLS!!!
    XTVinv = t(X * N)
    beta0 = solve(XTVinv %*% X) %*% (XTVinv %*% Z) ### this parenthesis is also helpful to speed up

    ## get dispersion estimates, and restrict a bit to bound away from 0/1.
    phiHat = (sum( (Z - X %*% beta0)^2 * N) - (n - p)) * n / (n - p) / sum(N-1)
    phiHat = min(max(c1, phiHat),1-c1)

    ## Shrinkage phiHat a bit --- how to do this???

    ## second round of regression.
    XTVinv = t(X * (N/(1+(N-1)*phiHat)))  ###t(X)%*%VInv
    XTVinvX.inv = solve(XTVinv %*% X)
    beta0 = solve(XTVinv %*% X) %*% (XTVinv %*% Z)
    se.beta0 = sqrt(diag(XTVinvX.inv))

    ## return. I'll flatten the var/cov matrix for easy storing.
    list(beta0=beta0, se.beta0=se.beta0, var.beta0 = as.vector(XTVinvX.inv), phi=phiHat)
}

##############################################################
### hypothesis testing function
##############################################################
DMLtest.multiFactor <- function(DMLfit, coef=2, term, Contrast) {
    ## figure out index of the factor to be tested
    ## coef = find.factor(DMLfit$design, DMLfit$formula, factor)
    ## check inputs
    flag = 0
    if(!missing(coef))
        flag = flag + 1
    if(!missing(term))
        flag = flag + 1
    if(!missing(Contrast))
        flag = flag + 1

    if(flag == 0)
        stop("Must specify one of the following parameter for testing: coef, term, or Contrast.\n")
    if(flag > 1)
        stop("You can only specify one of the following parameter for testing: coef, term, or Contrast.\n")

    if(!missing(coef)) { # specified coef
        res = DMLtest.multiFactor.coef(DMLfit, coef)
    } else if(!missing(term)) { # specify term
        ## create a contrast matrix for testing the term
        Contrast = makeContrast(DMLfit, term)
        ## testing
        res = DMLtest.multiFactor.Contrast(DMLfit, Contrast)

    } else if(!missing(Contrast)) { # specify contrast
        ## check contrast matrix
        if( nrow(Contrast) != ncol(DMLfit$X) )
            stop("Input Contrast matrix has wrong dimension: its number of rows must match the number of columns of the design matrix.\n")

        ## testing
        res = DMLtest.multiFactor.Contrast(DMLfit, Contrast)

    }

    class(res)[2] = "DMLtest.multiFactor"
    invisible(res)

}

##############################################################
## Hypothesis testing when specify a coef for testing.
## This only tests one column in the design matrix.
## Wald test will be used.
##############################################################

DMLtest.multiFactor.coef <- function(DMLfit, coef) {
    if(is.character(coef)) {
        tmp = which(colnames(DMLfit$X) == coef)
        if(length(tmp) == 0)
            stop(paste0("Can't find terms to be tested: ", coef,
                        ". Make sure it matches a column name in design matrix."))
        coef = tmp
    }

    ## hypothesis testing
    p = ncol(DMLfit$X)
    fit = DMLfit$fit
    betas = fit$beta[,coef]
    ## take out SE estimates from var/cov matrix
    tmp = t(apply(fit$var.beta, 1, function(x) diag(matrix(x, ncol=p))))
    ses = sqrt(tmp[,coef])

    ## Wald test, get p-values and FDR
    stat = betas / ses
    pvals = 2*pnorm(-abs(stat))  #2*(1- pnorm(abs(stat)))
    fdrs = p.adjust(pvals, method="BH")

    ## return a data frame
    gr = DMLfit$gr
    res = data.frame(chr=seqnames(gr), pos=start(gr), stat, pvals, fdrs)
    invisible(res)
}

##############################################################
## Hypothesis testing when specify a contrast matrix.
## This tests multiple columns in the design matrix.
## F-test will be used.
##############################################################

DMLtest.multiFactor.Contrast <- function(DMLfit, Contrast) {
    p = ncol(DMLfit$X)
    fit = DMLfit$fit
    betas = fit$beta
    ## A^T * beta
    Abeta = betas %*% Contrast

    ## loop through CpG sites -- have to do this since the var/cov matrices of the beta estimates
    ## are different for each site
    stat = rep( NA, nrow(betas) )
    for( i in 1:nrow(betas) ) {
        Sigma = matrix(fit$var.beta[i,], ncol=p)
        tmp = solve(t(Contrast) %*% Sigma %*% Contrast)
        thisAbeta = Abeta[i,,drop=FALSE]
        stat[i] = thisAbeta %*% tmp %*% t(thisAbeta)
    }

    ## get the sign of the contrast if there's only one contrast.
    ## This is to be added to test statistics
    ## When Contrast has multiple columns, there won't be a sign for test statistics.
    if(ncol(Contrast) == 1)
        signs = sign(betas %*% Contrast)
    else signs = 1

    ## get p-values. Stat follows F_{r, R}
    ## I found that using F distribution, the p-values are pretty large.
    ## Use sqrt(f) and normal gives much smaller p-values,
    ## and this is consistent with the Wald test in two-group comparison.
    r = ncol(Contrast)
##     R = nrow(DMLfit$X) - ncol(DMLfit$X)
##     stat = stat / r
##     pvals = 1 - pf(stat, r, R)
    stat = sqrt(stat / r) * signs
    pvals = 2*pnorm(-abs(stat))
    fdrs = p.adjust(pvals, method="BH")

    ## return a data frame
    gr = DMLfit$gr
    res = data.frame(chr=seqnames(gr), pos=start(gr), stat, pvals, fdrs)
    attr(res, "Contrast") = Contrast
    invisible(res)

}


##############################################################
## make contrast matrix given a model and a term to be tested
## Input term can be a vector (testing multiple terms)
##############################################################
makeContrast <- function(fit, term) {
    formula.terms = attr(terms(fit$formula), "term.labels")
    ix = match(term, formula.terms)
    if( any(is.na(ix)) )
        stop("Some term(s) to be tested can't be found in the formula.\n")

    ## make contrast matrix. All columns in the design matrix related to
    ## the provided term (including interactions) should be tested.
    allcolnam = colnames(fit$X)
    ix.term = NULL
    ixcol = attr(fit$X, "assign")
    for( t in term ) {
        iii = which(formula.terms == t)
        ix.term = c( ix.term, which(ixcol==iii) )
    }

    ## make matrix.
    L = matrix(0, ncol=ncol(fit$X), nrow=length(ix.term))
    for(i in 1:nrow(L))
        L[i, ix.term[i]] = 1

    return(t(L))
}

### function to compute normalization factors for an object of SeqCountSet
### I'm using a naive method now.
estNormFactors <- function(seqData, method=c("lr", "total", "quantile", "median")) {
  if(!is(seqData, "SeqCountSet"))
    stop("Input must be an object of SeqCountSet class!")

  method=match.arg(method)
  X <- exprs(seqData)
  if(method=="lr")
      k=estNormFactors.lr(X)
  if(method=="quantile")
      k=estNormFactors.quantile(X)
  else if(method=="total")
      k=estNormFactors.total(X)
  else if(method=="median")
      k=estNormFactors.median(X)

  normalizationFactor(seqData)=k
  seqData
}

estNormFactors.quantile <- function(X) {
  k=apply(X, 2, function(x) quantile(x[x>0], 0.75))
  k/min(k)
}

estNormFactors.total <- function(X) {
  k=colSums(X)
  k/min(k)
}

estNormFactors.median <- function(X) {
  k=apply(X, 2, median)
  k/min(k)
}

estNormFactors.lr <- function(X) {
    ss <- colSums(X)
    ix <- which.min(ss)## use this one as basis
    X0 <- X[,ix]
    k <- rep(1, ncol(X))

    for(i in 1:ncol(X)) {
        ii <- X0>0 & X[,i]>0
        lr <- log(X0[ii] / X[ii,i])
        ## trim 5%
        qq <- quantile(lr, c(0.05, 0.95))
        lr2 <- lr[lr>qq[1] & lr<qq[2]]
        k[i] <- exp(-median(lr2))
    }
    k[ix] <- 1
    k
}

## likihood ratio test.
## This works for two group comparison only!!
lrTest <- function(seqData) {
  ## compute EY under null
  Y=exprs(seqData)
  k=normalizationFactor(seqData)
  if(is.matrix(k))
    Y2=Y/k
  else
    Y2=sweep(Y, 2, k, FUN="/")
  muY0=sweep(matrix(rep(rowMeans(Y2),ncol(Y)),ncol=ncol(Y)),  2, k, FUN="*")
  
  ## EY under alternative
  design=as.factor(pData(phenoData(seqData))$designs)
  k=normalizationFactor(seqData)
  X=makeDesign(design, k)
  muY1 = calc.expY(Y+0.5, X)

  ## likelihoods
  phi=dispersion(seqData)
  l0=rowSums(dnbinom(Y, size=1/phi, mu=muY0, log=TRUE))
  l1=rowSums(dnbinom(Y, size=1/phi, mu=muY1, log=TRUE))
  lr=l1-l0 ## some are less than 0??
  
  pval=1-pchisq(lr, df=1)
  list(lr=lr, pval=pval)
}


                    

### dispersion shrinkage with multiple factor design
## Assuming dispersion is not dependent on means at this time
estDispersion.multiFactor <- function(seqData) {
  Y = exprs(seqData)
  k = normalizationFactor(seqData)
  
  ## design matrix
  design = as.matrix(pData(seqData))
  ## incorporate size factors
  Y2 = sweep(Y, 2, k, FUN="/")

  ## 1. Run olr, using log(counts) as outcomes, obtain E[Y]
  Y2=log(Y2+0.5)
  muY = exp(calc.expY(Y2, design))
  
  ## 2.  estimate gene specific phi
  z=Y^2-Y
  z=sweep(z, 2, k^2, FUN="/")
  phi.g=rowSums(z)/rowSums(muY^2)-1
  phi.g[phi.g<1e-8]=NA

  ## 3. shrinkage 
  ## first estimate hyper parameters
  s=rowMeans(Y)
  ii=s>=5
  phi.g0=phi.g[ii]
  lphi.g0=log(phi.g0)
  mu=median(lphi.g0, na.rm=TRUE) ## it seems mu is under estimated when var(OD) is small
  tau=sqrt((IQR(lphi.g0, na.rm=TRUE) / 1.349)^2)
  ## scale tau down a little bit. This needs more investigation,
  ## but it seems this doesn't affect the results much.
  tau = tau * 0.7
  
  ## NR procedure to do shrinkage
  max.value = max(lphi.g0,10, na.rm=TRUE)
  ## objective function (penalized likelihood). Shrink in the log scale to geometric means
  nsamples = nrow(design)
  get.phi <- function(dat){
    y=dat[1:nsamples]
    Ey=dat[nsamples+(1:nsamples)]
    
    obj=function(phi) {
      alpha=1/phi
      tmp1=1/(1+Ey*phi)
      tmp2=1-tmp1
      -(sum(lgamma(alpha+y)) - nsamples*lgamma(alpha) + alpha*sum(log(tmp1)) + sum(y*log(tmp2)) +
        dnorm(log(phi), mean=mu, sd=tau, log=TRUE))
    }
    return(optimize(obj, interval=c(0.01, max.value))$minimum)
  }
  tmp=cbind(Y, muY)
  phi.hat=apply(tmp,1,get.phi)

  dispersion(seqData) = phi.hat
  seqData
}

### functions to estimate and shrink gene specific over dispersion
## this works for single factor design only at this time!!!
estDispersion <- function(seqData, trend=FALSE) {
  if(!is(seqData, "SeqCountSet"))
    stop("Input must be an object of SeqCountSet class!")
  
  ## design matrix
  design <- pData(seqData)
  if(ncol(design) == 1) { ## single factor design
    if(trend) 
      seqData=estDispersion.trend(seqData)
    else
      seqData=estDispersion.notrend(seqData)
  } else { ## multiple factor
    seqData=estDispersion.multiFactor(seqData)
  }
  
  seqData
}

########################################################
##
## function to call DMRs from the DML test results
##
########################################################

callDMR <- function(DMLresult, delta=0, p.threshold=1e-5,
                    minlen=50, minCG=3, dis.merge=100, pct.sig=0.5) {
    ## remove the NA entries
    ix.keep = !is.na(DMLresult$stat)
    if(mean(ix.keep) < 1)  ## with NA entries in the results
        DMLresult = DMLresult[ix.keep,]

    flag.multifactor = FALSE
    if(class(DMLresult)[2] == "DMLtest.multiFactor")
        flag.multifactor = TRUE

    if(dis.merge > minlen)
        dis.merge = minlen

    ## deal with delta
    if( delta > 0 ) {
        if(flag.multifactor) { # multifactor, doesn't support delta
            stop("The test results is based on multifactor design, 'delta' is not supported")
        }
        p1 <- pnorm(DMLresult$diff-delta, sd=DMLresult$diff.se) ## Pr(delta.mu > delta)
        p2 <- pnorm(DMLresult$diff+delta, sd=DMLresult$diff.se, lower.tail=FALSE) ## Pr(-delta.mu < -delta)
        postprob.overThreshold <- p1 + p2
        DMLresult <- data.frame(DMLresult, postprob.overThreshold=postprob.overThreshold)
        scores <- 1 - postprob.overThreshold
        ## modify the test statistics according to postprob - this seems make the results worse
        ##        DMLresult[,"stat"] <- qnorm(scores) * sign(DMLresult[,"stat"])
    } else {
        scores <- DMLresult$pval
    }

    ## bump finding
    dmrs <- findBumps(DMLresult$chr, DMLresult$pos, scores,
                      cutoff=p.threshold, sep=5000, dis.merge=dis.merge,
                      pct.sig=pct.sig, minCG=minCG)

    ## compute average methylation levels in two groups
    if(is.null(dmrs)) {
        warning("No DMR found! Please use less stringent criteria. \n")
        return(NULL)
    }

    ## Looping seems to be very slow.
    nCG <- dmrs[,"idx.end.global"] - dmrs[,"idx.start.global"] + 1
    ix.good <- dmrs$length>minlen & nCG>minCG
    if(sum(ix.good) == 0) {
        warning("No DMR found! Please use less stringent criteria. \n")
        return(NULL)
    }
    dmrs <- dmrs[ix.good,]
    nCG <- dmrs[,"idx.end.global"] - dmrs[,"idx.start.global"] + 1

    ## create final result data frame
    if(flag.multifactor) { ## multifactor
        areaStat = rep(0, nrow(dmrs))
        for(i in 1:nrow(dmrs)) { ## this part is kind of slow when number of DMRs is large
            ii=dmrs[i,"idx.start.global"]:dmrs[i,"idx.end.global"]
            areaStat[i] = sum(DMLresult[ii,"stat"])
        }
        result <- data.frame(dmrs[,1:4], nCG=nCG, areaStat=areaStat)

    } else { ## single factor
        meanMethy1 = meanMethy2 = areaStat = rep(0, nrow(dmrs))
        for(i in 1:nrow(dmrs)) { ## this part is kind of slow when number of DMRs is large
            ii=dmrs[i,"idx.start.global"]:dmrs[i,"idx.end.global"]
            meanMethy1[i] = mean(DMLresult[ii,"mu1"])
            meanMethy2[i] = mean(DMLresult[ii,"mu2"])
            areaStat[i] = sum(DMLresult[ii,"stat"])
        }

        result <- data.frame(dmrs[,1:4], nCG=nCG, meanMethy1, meanMethy2,
                             diff.Methy=meanMethy1-meanMethy2, areaStat=areaStat)
    }


    ## sort by areaStat
    ix = sort(abs(result$areaStat), decreasing=TRUE, index.return=TRUE)$ix
    result[ix,]
}


#######################################################
## seperate data into blocks according to chr and pos
#######################################################
findRegion <- function(chr, pos, sep=1000) {
    pos.diff <- abs(c(as.integer(0), diff(pos)))
    idx.jump <- which(pos.diff>sep)
    regions <- rbind(c(1, idx.jump), c(idx.jump-1, length(pos)))
    regions
}

#######################################################
## Bump finding, given score and cutoff.
## This is slow. Need to rewrite in C.
#######################################################
findBumps <- function(chr, pos, x, cutoff, sep=1000, dis.merge=200, pct.sig=0.3, minCG) {
    flag <- as.numeric(x<cutoff)
    if(sum(flag) == 0) ## none
        return(NULL)

    flag[is.na(flag)]=FALSE
    ## find regions
    regions <- findRegion(chr, pos, sep)
    ## loop on regions
    initn <- 100000 ## initialze number of DMRs. Allocate enough rows to start with.
    result <- data.frame(chr=rep("chr1",initn), start=rep(0,initn),
                         end=rep(0, initn), length=rep(0, initn),
                         idx.start.global=rep(0, initn),
                         idx.end.global=rep(0, initn))
    levels(result[,1]) <- unique(chr)
    result.idx <- 0
    for(i in 1:ncol(regions)) {
        idx <- regions[1,i]:regions[2,i]
        if(length(idx) <= minCG) next
        pos.region <- pos[idx]
        nn <- length(idx)
        flag.region <- flag[idx]
        ## get start/end position
        startidx <- which(flag.region[-nn]==0 & flag.region[-1]==1)+1
        if(flag.region[1]==1)
            startidx <- c(1, startidx)
        if(length(startidx)==0)
            next
        endidx <- which(flag.region[-nn]==1 & flag.region[-1]==0)
        if(flag.region[nn]==1)
            endidx <- c(endidx, nn)

        ## skip if there are less than minCount probes
##         idx.keep <- (endidx-startidx+1)>=minCG
##         startidx <- startidx[idx.keep]
##         endidx <- endidx[idx.keep]
##         if(length(endidx)==0) next

        ## merge if they are really close
        nbump <- length(startidx)
        if(nbump>1) {
            bumppos <- cbind(pos[idx][startidx], pos[idx][endidx])
            dis <- bumppos[-1,1]>(bumppos[-nbump,2]+dis.merge)
            idx.start <- which(c(1,dis)==1)
            idx.end <- which(c(dis,1)==1)
            ## merged
            startidx <- startidx[idx.start]
            endidx <- endidx[idx.end]
        }

        ## after merging, make sure certain % of CG sites being significant is enough.
        ## this is not easy!!!
        ix.good <- NULL
        x.thisregion <- x[idx]
        for(ibump in 1:length(startidx)) {
            ii <- startidx[ibump]:endidx[ibump]
            pp <- x.thisregion[ii] < cutoff
            if(mean(pp) > pct.sig)
                ix.good <- c(ix.good, ibump)
        }
        if(length(ix.good) == 0) next
        else {
            startidx <- startidx[ix.good]
            endidx <- endidx[ix.good]
        }

        nbump <- length(startidx)
        ll <- pos.region[endidx] - pos.region[startidx] + 1
        tmpn <- length(ll)

        ## make result
        result[result.idx+(1:tmpn),] <- data.frame(chr=as.character(chr[idx][startidx]),
                                                   start=pos[idx][startidx],
                                                   end=pos[idx][endidx], length=ll,
                                                   idx.start.global=idx[startidx],
                                                   idx.end.global=idx[endidx])
        result.idx <- result.idx + tmpn
    }

    if(result.idx >= 1)
        result <- result[1:result.idx,]
    else return(NULL)
    result
}

################################################
## A wrapper function for performing RNA-seq DE.
## Use all default parameter values.
################################################
DSS.DE <- function(counts, design) {
    ## some error checkings
    if(! all(unique(design) %in% c(0,1)) )
        stop("design must be a vector of 0 and 1.")
    if(length(design) != ncol(counts))
        stop("length of design doesn't match the number of columns of count matrix.")

    seqData = newSeqCountSet(counts, design)
    seqData = estNormFactors(seqData)
    seqData = estDispersion(seqData)
    result = waldTest(seqData, 0, 1)
    result
}

##
## We need the splines library, so we load it.
##

## .onLoad <- function(lib, pkg) require(splines)

locmle <-
function(z, xlim , Jmle = 35, d = 0, s = 1, ep = 1/100000, sw = 0, Cov.in)
{
        ## uses z-values in [-xlim,xlim] to find mles for p0,del0,sig0
        ## Jmle number of iterations, beginning at (del0,sig0)=(d,s)
        ## sw=1 returns correlation matrix
        N = length(z)
        if (missing(xlim)) {
          if (N>500000) b = 1
          else b=4.3 * exp(-0.26*log(N,10)) 
          xlim=c(median(z),b*diff(quantile(z)[c(2,4)])/(2*qnorm(.75)))
        }
        aorig=xlim[1]-xlim[2]
        borig=xlim[1]+xlim[2]
        z0=z[which(z>=aorig & z<=borig)]
        N0 = length(z0)
        Y = c(mean(z0), mean(z0^2))
        that = N0/N
        ######## find mle estimates ###########################
        for(j in 1:Jmle) {
                bet = c(d/s^2, -1/(2 * s^2))
                aa = (aorig - d)/s
                bb = (borig - d)/s
                H0 = pnorm(bb) - pnorm(aa)
                fa = dnorm(aa)
                fb = dnorm(bb)
                H1 = fa - fb
                H2 = H0 + aa * fa - bb * fb
                H3 = (2 + aa^2) * fa - (2 + bb^2) * fb
                H4 = 3 * H0 + (3 * aa + aa^3) * fa - (3 * bb + bb^3) * fb
                H = c(H0, H1, H2, H3, H4)
                r = d/s
                I = matrix(rep(0, 25), 5)
                for(i in 0:4)
                        I[i + 1, 0:(i + 1)] = choose(i, 0:i)
                u1 = s^(0:4)
                II = pmax(row(I) - col(I), 0)
                II = r^II
                I = u1 * (I * II)
                E = as.vector(I %*% H)/H0
                E1 = E[2]
                E2 = E[3]
                E3 = E[4]
                E4 = E[5]
                mu = c(E1, E2)
                V = matrix(c(E2 - E1^2, E3 - E1 * E2, E3 - E1 * E2, E4 -
                        E2^2), 2)
                bett = bet + solve(V, Y - mu)/(1 + 1/j^2)
                if(bett[2]>0)bett=bet+.1*solve(V, Y - mu)/(1 + 1/j^2)
                if (is.na(bett[2])) break
                else if (bett[2]>=0) break
                d =  - bett[1]/(2 * bett[2])
                s = 1/sqrt(-2 * bett[2])
                if(sum((bett - bet)^2)^0.5 < ep)
                        break
        }
        if (is.na(bett[2])) {
          mle = rep(NA, 6)
          Cov.lfdr = NA
          Cor = matrix(NA, 3, 3)
        }
        else if (bett[2] >=0)  {
          mle = rep(NA, 6)
          Cov.lfdr = Cov.out = NA
          Cor = matrix(NA, 3, 3)
        }
        else {
          aa = (aorig - d)/s
          bb = (borig - d)/s
          H0 = pnorm(bb) - pnorm(aa)
          p0 = that/H0
          ############  sd calcs ###########################
          J = s^2 * matrix(c(1, 0, 2 * d, s), 2)
          JV = J %*% solve(V)
          JVJ = JV %*% t(J)
          mat2 = cbind(0, JVJ/N0)
          mat1 = c((p0 * H0 * (1 - p0 * H0))/N, 0, 0)
          mat = rbind(mat1, mat2)
          h = c(H1/H0, (H2 - H0)/H0)
          matt = c(1/H0,  - (p0/s) * t(h))
          matt = rbind(matt, cbind(0, diag(2)))
          C = matt %*% (mat %*% t(matt))
          mle = c(p0, d, s, diag(C)^0.5)
          if(sw == 1) {
                sd = mle[4:6]
                Co = C/outer(sd, sd)
                dimnames(Co) = list(c("p0", "d", "s"), c("p0", "d", "s"))
                Cor = Co[c(2, 3, 1), c(2, 3, 1)]
          }
          if (!missing(Cov.in)) {
            i0 = which(Cov.in$x > aa & Cov.in$x < bb)
            Cov.out = loccov(N, N0, p0, d, s, Cov.in$x, Cov.in$X, Cov.in$f,
                             JV, Y, i0, H, h, Cov.in$sw)
          }
        }
        names(mle) = c("p0", "del0", "sig0", "sd.p0", "sd.del0", "sd.sig0")
        mle = mle[c(2, 3, 1, 5, 6, 4)]
        out = list(mle=mle)
        if (sw==1) {
          Cor = list(Cor = Cor)
          out = c(out, Cor)
        }
        if (!missing(Cov.in)) {
          if (Cov.in$sw == 2) {
            pds. = list(pds.=Cov.out)
            out = c(out, pds.)
          }
          else if (Cov.in$sw == 3) {
            Ilfdr = list(Ilfdr=Cov.out)
            out = c(out, Ilfdr)
          }
          else {
            Cov.lfdr = list(Cov.lfdr=Cov.out)
            out = c(out, Cov.lfdr)
          }
        }
        if ((sw==1) | !missing(Cov.in)) return(out)
        else return(mle)
}

loccov = function(N, N0, p0, d, s, x, X, f, JV, Y, i0, H, h, sw) {
  M = rbind(1, x - Y[1], x^2 - Y[2])
  if (sw==2) {
    K = length(x)
    K0 = length(i0)    
    toprow = c(1 - N0/N, -t(h) %*% JV / s)
    botrow = cbind(0, JV / p0)
    mat = rbind(toprow, botrow)
    M0 = M[,i0]
    dpds.dy0 = mat %*% M0 / N / H[1]
    dy0.dy = matrix(0, K0, K)
    dy0.dy[,i0] = diag(1, K0)
    dpds.dy = dpds.dy0 %*% dy0.dy
    rownames(dpds.dy) = c("p", "d", "s")
    return(dpds.dy)
  }
  else {
    xstd = (x - d)/s
    U = cbind(xstd - H[2]/H[1], xstd^2 - H[3]/H[1])
    M[,-i0] = 0
    dl0plus.dy = cbind(1 - N0/N, U %*% JV / s) %*% M /N/H[1]/p0  
    G <- t(X) %*% (f * X)
    dl.dy = X %*% solve(G) %*% t(X)
    dlfdr.dy = dl0plus.dy - dl.dy
    if (sw==3) return(dlfdr.dy)
    else {
      Cov.lfdr = dlfdr.dy %*% (f * t(dlfdr.dy))
      return(Cov.lfdr)
    }
  }
}

loccov2 = function(X, X0, i0, f, ests, N) {
        d = ests[1]
        s = ests[2]
        p0 = ests[3]
        theo = I(ncol(X0)==1)
        Xtil <- X[i0,]
        X0til <- X0[i0,]
        G <- t(X) %*% (f * X)
        G0 <- t(X0til) %*% X0til
        B0 <- X0 %*% (solve(G0) %*% t(X0til)) %*% Xtil
        C <- B0 - X
        Ilfdr = C %*% solve(G, t(X))
        Cov <- C %*% solve(G) %*% t(C)
        if (theo)
          D = matrix(1,1,1)
        else
          D = matrix(c(1, 0, 0, d, s^2, 0, s^2 + d^2, 2 * d * s^2, s^3), 3)
	gam. = solve(G0, t(X0til)) %*% (Xtil %*% solve(G, t(X)))
	pds. = D %*% gam.
        if (theo) pds. = rbind(pds., matrix(0, 2, nrow(X)))
	pds.[1,] = pds.[1,] - 1/N
	m1 = pds. %*% f
	m2 = pds.^2 %*% f
	stdev = as.vector(sqrt(m2 - m1^2/N))
	stdev[1] = p0 * stdev[1]
        pds.[1,] = p0 * pds.[1,]
        rownames(pds.) = c("p", "d", "s")
        list(Ilfdr = Ilfdr, pds.=pds., stdev=stdev, Cov=Cov)
}

setClass( "SeqCountSet",
         contains = "ExpressionSet",
         representation = representation(
           normalizationFactor="numeric",
           dispersion="numeric"
           )
         )

newSeqCountSet <- function(counts, designs, normalizationFactor=NULL,featureData = NULL ) {
  ## some error checking stuff

  if( is.null( featureData ) )
    featureData <- annotatedDataFrameFrom( counts, byrow=TRUE )

  ## check input normalizationFactor
  if(is.null(normalizationFactor)) { ## not provided, calculate it
      normalizationFactor <- estNormFactors.lr(counts)
  }
  else { # with normalizationFactor provided
    if(is(normalizationFactor, "numeric")) { ## input is vector
      if(length(normalizationFactor) != ncol(counts))
        stop("Input normalizationFactor has wrong length.")
    } else if(is.matrix(normalizationFactor)) { ## matrix normalization factor
      ## check dimension
      if(any(dim(counts) != dim(normalizationFactor)))
        stop("Input normalizationFactor has wrong dimension.")
      ## convert to vector
      normalizationFactor=as.numeric(normalizationFactor)
    } else { ## other things
      stop("Wrong input normalizationFactor, must be a numeric vector or matrix.")
    }
  }

  ## work on input design matrix. Carefully about the input design.
  if(is(designs, "vector") ) { # single factor design
    designs <- as.data.frame(designs )
    colnames(designs)="designs"
  } else  if( is( designs, "matrix" ) ) { ## multiple factor design
    if(ncol(designs) == 1) { ## still single factor
      designs <- as.data.frame(designs )
      colnames(designs)="designs"
    } else { ## input is a data frame
      designs <- as.data.frame(designs )
    }
  }
  ## if there's no row names in designs, assume it has the same sequence as the columns in count matrix
  if( is.null(rownames(designs)) & !is.null(colnames(counts)) ) {
    rownames(designs)=colnames(counts)
  }

  if( is( designs, "data.frame" ) || is( designs, "AnnotatedDataFrame" ) ) {
    stopifnot( nrow( designs ) == ncol( counts ) )
    designs <- as( designs, "AnnotatedDataFrame" )
  }


  res <- new("SeqCountSet",
             exprs = counts,
             phenoData = designs,
             featureData = featureData,
             normalizationFactor=normalizationFactor,
             dispersion=numeric(0)
             )
  res

}

### normalization factor
setGeneric("normalizationFactor", function(object) standardGeneric("normalizationFactor"))
setGeneric("normalizationFactor<-", function(object, value) standardGeneric("normalizationFactor<-"))

setMethod("normalizationFactor", signature(object="SeqCountSet"),
          function(object) {
            f=object@normalizationFactor
            n=ncol(exprs(object))
            if(length(f)==n) { ## size factor as a vector
              res=f
            } else { ## a matrix
              res=matrix(f, ncol=n)
            }
            res
          })

setReplaceMethod("normalizationFactor", signature(object="SeqCountSet", value="numeric"),
                 function(object, value ) {
                   if(length(value) != ncol(exprs(object)))
                     stop("wrong length for normalization factor vector!")
                   object@normalizationFactor <- value
                   object
                 })

setReplaceMethod("normalizationFactor", signature(object="SeqCountSet", value="matrix"),
                 function(object, value ) {
                   if(any(dim(value) != dim(exprs(object))))
                     stop("wrong dimension for normalization factor matrix!")
                   object@normalizationFactor <- as.numeric(value)
                   object
                 })

## for gene specific over-dispersion
setGeneric("dispersion", function(object) standardGeneric("dispersion"))
setGeneric("dispersion<-", function(object, value) standardGeneric("dispersion<-"))

setMethod("dispersion", signature(object="SeqCountSet"),
          function(object) {
            object@dispersion
          })

setReplaceMethod("dispersion", signature(object="SeqCountSet", value="numeric"),
                 function(object, value ) {
                   if(length(value) != nrow(exprs(object)))
                     stop("wrong length for dispersion vector!")
                   object@dispersion <- value
                   object
                 })

##### Functions to estimate and shrink dispersions
## when there's no trend between dispersion and expression
estDispersion.notrend <- function(seqData) {
  design=as.factor(pData(phenoData(seqData))$designs)
  k=normalizationFactor(seqData)

  ## make design matrix
  X=makeDesign(design, k)

  ## compute the expected Y
  Y=exprs(seqData)+0.5
  muY = calc.expY(Y, X)

  ## estimate gene specific phi
  df=ncol(Y)-ncol(X)
  phi.g = est.dispersion(Y, k, design)

  ## shrink phi
  phi.hat = shrink.dispersion.notrend(phi.g, Y, muY, k, design)
  dispersion(seqData) = phi.hat

  seqData
}

### function to shrink dispersion
## Y is normalized by size factors and designs.
shrink.dispersion.notrend <- function(phi.g, Y, muY, k, design) {
  nsamples=ncol(Y)
  ngenes=nrow(Y)
  phi.hat=rep(0,ngenes)

  ## estimate hyper parameters - this is tricky!!
  s=rowMeans(Y)
  ii=s>=5
  phi.g0=phi.g[ii]
  lphi.g0=log(phi.g0)
  ## It seems phi.g is a little under estimated.
  m0=median(lphi.g0, na.rm=TRUE) ## it seems m0 is under estimated when var(OD) is small
  sigma2.mar=(IQR(lphi.g0, na.rm=TRUE) / 1.349)^2
  ## estimate the part being over estimated and subtract.
  ## It seems I'm subtracting too much when OD is big!!
  sigma2.base=compute.baseSigma.notrend(exp(m0), Y[ii,], muY[ii,], k, design)
  sigma=sqrt(max(sigma2.mar-sigma2.base, 1e-2))

  ## NR procedure to do shrinkage
  max.value = max(lphi.g0,10, na.rm=TRUE)
  ## objective function (penalized likelihood)
  get.phi <- function(dat){
    y=dat[1:nsamples]
    Ey=dat[nsamples+(1:nsamples)]
    obj=function(phi) {
      alpha=1/phi
      tmp1=1/(1+Ey*phi)
      tmp2=1-tmp1
      -(sum(lgamma(alpha+y)) - nsamples*lgamma(alpha) + alpha*sum(log(tmp1)) + sum(y*log(tmp2)) -
        ((log(phi) - m0)^2) / (2*(sigma^2)) - log(phi) - log(sigma))
    }
    return(optimize(obj, interval=c(0.01, max.value))$minimum)
  }
  tmp=cbind(Y, muY)
  phi.hat=apply(tmp,1,get.phi)
  phi.hat
}

### compute hyperparameter sigma, when all genes have the same phi.
## this seems over estimated that. Need to think!!!
compute.baseSigma.notrend <- function(phi0, Y, muY,k, design) {
  n=length(phi0)
  m=ncol(Y)
  nsim=10
  res=rep(0, nsim)
  ## sample, ignore size factor. This is okay.
  k=rep(1, m)
  for(i in 1:nsim) {
    Ysim <- matrix(rnegbinom(length(Y), muY, phi=phi0), ncol=m)
    lag.sim=log(est.dispersion(Ysim, k, design))
    sigma=IQR(lag.sim, na.rm=TRUE) / 1.349
    res[i]=sigma^2
  }
  mean(res)
}

##### Functions to estimate and shrink dispersions
## when there IS trend between dispersion and expression
estDispersion.trend <- function(seqData) {
  design=as.factor(pData(phenoData(seqData))$designs)
  k=normalizationFactor(seqData)

  ## make design matrix
  X=makeDesign(design, k)

  ## compute the expected Y. Should this be under null???
  Y=exprs(seqData)+0.5
  muY = calc.expY(Y, X)

  ## estimate gene specific phi
  df=ncol(Y)-ncol(X)
  phi.g = est.dispersion(Y, k, design)

  ## estimate phi~expr trends
##   lexpr1=rowMeans(log(sweep(Y+0.5,2,k,FUN="/")))
##   lOD=log(phi.g)
##   ix=!is.na(lOD)
##   fit1=smooth.spline(lOD[ix]~lexpr1[ix], df=3, nknots=round(sum(ix)/50))
##   lOD.pred=predict(fit1,lexpr1)$y
  lOD.pred=est.trend(Y)
  ## shrink phi
  phi.hat = shrink.dispersion.trend(phi.g, lOD.pred, Y, muY, k, design)
  dispersion(seqData) = phi.hat

  seqData
}


### function to shrink dispersion
## Y is normalized by size factors and designs.
## lOD.pred is predicted dispersion in log scale.
shrink.dispersion.trend <- function(phi.g, lOD.pred, Y, muY, k, design) {
  nsamples=ncol(Y)
  ngenes=nrow(Y)
  phi.hat=rep(0,ngenes)

  ## estimate hyper parameters - this is tricky!!
  ## lOD.pred is the mean
  ## need to estimate tau^2
  lphi.g0=log(phi.g)
  lexpr1=rowMeans(log(sweep(Y,2,k,FUN="/")))
  lexpr.cut=2
  ix=lexpr1>lexpr.cut
  sigma2.mar=(IQR(lphi.g0[ix]-lOD.pred[ix], na.rm=TRUE) / 1.349)^2

  ## remove the amount of over-estimation
  sigma2.base=compute.baseSigma.trend(lOD.pred, Y, muY, k, design)
  sigma=sqrt(max(sigma2.mar-sigma2.base, 1e-2))

  ## The way to estimate mu and sigma needs some thinking.
  ## However it doesn't seem to make much differences.
  ## NR procedure to do shrinkage
  max.value = max(lphi.g0,10, na.rm=TRUE)
  ## objective function (penalized likelihood)
  get.phi <- function(dat){
    y=dat[1:nsamples]
    Ey=dat[nsamples+(1:nsamples)]
    mu.phi=dat[length(dat)]
    obj=function(phi) {
      alpha=1/phi
      tmp1=1/(1+Ey*phi)
      tmp2=1-tmp1
      -(sum(lgamma(alpha+y)) - nsamples*lgamma(alpha) + alpha*sum(log(tmp1)) + sum(y*log(tmp2)) -
        ((log(phi) - mu.phi)^2) / (2*(sigma^2)) - log(phi) - log(sigma))
    }
    return(optimize(obj, interval=c(0.01, max.value))$minimum)
  }

  tmp=cbind(Y, muY, lOD.pred)
  phi.hat=apply(tmp,1,get.phi)
  phi.hat
}

### compute hyperparameter sigma, when all genes have the same phi.
## this seems over estimated that. Need to think!!!
compute.baseSigma.trend <- function(lOD.pred, Y, muY,k, design) {
  phi0=exp(lOD.pred)
  n=length(phi0)
  m=ncol(Y)
  nsim=5
  res=rep(0, nsim)
  ## sample
  k=rep(1,m)
  lexpr.cut=2
  for(i in 1:nsim) {
    Ysim <- matrix(rnegbinom(length(Y), muY, phi=phi0), ncol=m)
    lexpr.sim=rowMeans(log(sweep(Ysim+0.5,2,k,FUN="/")))
    phi.sim = est.dispersion(Ysim, k, design)
    lOD.sim=log(phi.sim)
    lOD.pred.sim=est.trend(Ysim)
    ix=lexpr.sim>lexpr.cut
    sigma=IQR(lOD.sim[ix]-lOD.pred.sim[ix], na.rm=TRUE) / 1.349
    res[i]=sigma^2
  }
  mean(res)
}

###############################################
## functions for estimating trends
###############################################
scv <- function(x) var(x)/mean(x)^2

## trend estimation. Results are trend in log scale
est.trend <- function(X,xmin=1){
  ss=colSums(X); ss=ss/min(ss)
  X= sweep(X,2,ss,FUN="/")
  phi.hat1 = est.phi0(X)
  lexpr=rowMeans(log(X))
  lexpr.cut=2
  phi0a=median(phi.hat1[lexpr>lexpr.cut])
  SCV=apply(X,1,scv)
  ##phi0 is the m0, average phi0 assuming no trend
  binsize=0.2
  xmax=quantile(lexpr, 0.99, na.rm=TRUE)
  trend1=tapply(phi.hat1-phi0a,cut(lexpr,c(seq(xmin,xmax,binsize),Inf)),median)
  trend1=-isoreg(-trend1)$yf
  fit1=smooth.spline(trend1~seq(xmin,xmax,binsize),df=3)
  pred1=predict(fit1,lexpr)$y
  pred1[lexpr>xmax]=predict(fit1,xmax)$y
  pred1[lexpr<xmin]=pmax(predict(fit1,xmin)$y,pred1[lexpr<xmin])
  m0.adj=pred1+phi0a
  log(m0.adj)
}

########################################################################
## some utility functions for estimating and shrinking dispersions
########################################################################

### function to create design matrix based on design and size factor.
## Works for single factor now.
makeDesign <- function(design, k) {
  allfactors <- levels(design)
  X=matrix(0, nrow=length(k), ncol=length(allfactors))
  for(i in 1:length(allfactors)) {
    idx=design==allfactors[i]
    X[idx,i]=1
  }
  if(is.matrix(k))
    stop("Matrix normalization factor is still under development!")
  else
    X=sweep(X, 1, k, FUN="*")
  X
}

## Estimate gene specific over dispersion using MOM.
## This considers the experimental design
est.dispersion <- function(Y, k, design) {
  ## adjust by size factor
  if(is.matrix(k))
    Y2=Y/k
  else
    Y2=sweep(Y, 2, k, FUN="/")

  ## transform
  z=Y^2-Y
  z=sweep(z, 2, k^2, FUN="/")

  ## first find group labels and compute group means.  works only for two groups now
  ll = levels(design)
  mus = matrix(0, nrow=nrow(Y), ncol=ncol(Y))

  for(i in 1:length(ll)) {
      idxA = design==ll[i]
      muA = rowMeans(Y2[,idxA,drop=FALSE])
      mus[,idxA] = muA
  }
##   idxB = design==ll[2];
##   muB = rowMeans(Y2[,idxB])
##   mus = matrix(0, nrow=nrow(Y), ncol=ncol(Y))
##   mus[,idxA] = muA;
##   mus[,idxB] = muB
  phi = rowSums(z) / rowSums(mus^2) - 1
##   phiA=rowMeans(z[,idxA])/(muA^2) - 1
##   phiB=rowMeans(z[,idxB])/(muB^2) - 1
##   phi=(phiA+phiB)/2
  phi[phi<1e-8]=NA
  phi
}

## Estimate gene specific over dispersion using MOM.
## This assumes all data are from the same distribution
est.phi0 <- function(Y) {
  k=colSums(Y) ## naive estimation of size factor
  k=k/min(k)
  Y2=sweep(Y, 2, k, FUN="/")
  mm0=rowMeans(Y2)
  idx0=mm0==0
  res=rep(0, nrow(Y))
  res[idx0]=NA

  ix=mm0>0
  Y2=Y2[ix,]
  m=rowMeans(Y2)
  v=apply(Y2, 1, var)
  phi.g = (v-m)/m^2
  res[!idx0]=phi.g
  res[res<1e-5]=1e-5

  ##n0=rowMeans(Y==0)
  ##ix.valid=mm0>10 & n0<=0.2
  ##list(phi=res, ix.valid=ix.valid)
  res
}


### function to compute expected value for Y
calc.expY <- function(Y, X) {
  b=(Y %*% X) %*% solve(t(X)%*%X)
  muY=b %*% t(X)
  muY
}

###################################
## some plotting functions for DMR
###################################


###################################
## plot one DMR.
###################################

showOneDMR <- function(OneDMR, BSobj, ext=500, ylim=c(0,1)) {
    ## get chr, position and counts - could this be slow if data is huge??
    allchr = as.character(seqnames(BSobj))
    allpos = start(BSobj)
    X = getBSseq(BSobj, "M");  N = getBSseq(BSobj, "Cov")
    ## locate the data for plotting
    chr = as.character(OneDMR$chr)
    ix.chr = which(allchr==chr)
    thispos = allpos[ix.chr]
    thisN = N[ix.chr,]
    thisX = X[ix.chr,]
    xlim = c(OneDMR$start-ext, OneDMR$end+ext)
    ix1 = which(thispos<=xlim[2] & thispos>=xlim[1])
    ## plot it
    nSample = ncol(X)
    if(nSample > 2) {
        y.cex=0.66
    } else  y.cex=1

    sNames = sampleNames(BSobj)
    par(mfrow=c(nSample, 1), mar = c(2.5, 2.5, 1.6, 2.5), mgp = c(1.5, 0.5, 0))
    thisP = thisX/thisN
    maxN = max(thisN[ix1,])

    for(i  in 1:ncol(X)) {
        ## blue foreground, methylated, normalized to ylim
        ## plot(thispos[ix1], thisP[ix1,i], type="h", col="blue", lwd=1, xlab=chr, ylim=ylim, xlim=xlim,
        ## ylab="methyl%", main=sNames[i], col.lab="blue")
        plot(thispos[ix1], thisP[ix1,i], type="h", col="blue", axes=F, lwd=1.5,
             xlab='', ylab='', ylim=ylim, xlim=xlim,
             main=sNames[i])
        box(col="black")
        axis(1,)
        axis(2, col="blue", col.axis="blue")
        mtext(chr, side=1, line=1.33, cex=y.cex)
        mtext("methyl%", side=2, line=1.33, col="blue", cex=y.cex)

        ## black curve, total counts, normalized to ylim
        thisN.norm = thisN[ix1,i]/maxN * ylim[2]
        lines(thispos[ix1], thisN.norm, type="l", col="gray", lwd=1.5)
        ## Be careful in y-axis labels, especially when the counts are very low
        if(maxN >= 4) {
            axis(side=4, at=seq(0,ylim[2],length.out=5),
                 labels=round(seq(0, maxN, length.out=5)) )
        } else {
            axis(side=4, at=seq(0,ylim[2],length.out=maxN+1), labels=0:maxN)
        }

        mtext("read depth", side=4, line=1.33, cex=y.cex)
        ## plot a shaded region for DMR
        rect(OneDMR$start, ylim[1], OneDMR$end, ylim[2], col="#FF00001A", border = NA)
    }
}

## perform wald test
## This works for two group comparison only!
waldTest <- function(seqData, sampleA, sampleB, equal.var=FALSE,
                     fdr.method=c("BH", "locfdr"))  {

    fdr.method = match.arg(fdr.method)
    Y0 = exprs(seqData)
    idx1 = rowSums(Y0)>0 ## genes with some counts

    ## compute two group means
    Y = exprs(seqData) ## +0.1
    design = pData(phenoData(seqData))$designs
    k = normalizationFactor(seqData)
    if(is.matrix(k))
        Y2 = Y/k
    else
        Y2 = sweep(Y, 2, k, FUN="/")

    ## difference
    idxA = design==sampleA
    nA = sum(idxA)
    muA = rowMeans(Y2[,idxA,drop=FALSE])
    idxB = design==sampleB
    nB = sum(idxB)
    muB = rowMeans(Y2[,idxB, drop=FALSE])
    d = muA-muB
    ## variance
    phi.hat = dispersion(seqData)
    ## use common mu:
    mu0 = rowMeans(Y2)
    n = length(k)
    if(equal.var) {
        std = sqrt((mu0*sum(1/k[idxA])+ nA*mu0^2*phi.hat)/nA^2+
                     (mu0*sum(1/k[idxB])+nB*mu0^2*phi.hat)/nB^2)
    } else  {
        std = sqrt((muA*sum(1/k[idxA])+ nA*muA^2*phi.hat)/nA^2+ 
                     (muB*sum(1/k[idxB])+nB*muB^2*phi.hat)/nB^2)
    }

    stat = d / std
    stat[is.na(stat)] = 0
    pval = 2*(1-pnorm(abs(stat)))

    ## generate a data frame of reports
    lfc = log((muA+0.5)/(muB+0.5)) ## log fold change
    difExpr = muA-muB ## difference in expressions
    genes = featureData(seqData)
    if(!is.null(genes))
      genes=as(genes, "data.frame")
    result = data.frame(geneIndex=1:nrow(Y), muA=muA, muB=muB, 
                        lfc=lfc, difExpr=difExpr, stats=stat, 
                        pval=pval, genes)
    
    if(fdr.method == "BH") {
      fdr0 = p.adjust(pval[idx1], method="BH")
      fdr = pval
      fdr[idx1] = fdr0
      result = data.frame(result, fdr=fdr)
    }
    
    if(fdr.method == "locfdr") {
        ## FDR esimation using locfdr. This is only for genes with non-zero counts.
        ## Need to work on this more, since it's not very good.
        stat1 = stat[idx1]
        normstat = (stat1 - median(stat1)) / (IQR(stat1) / 1.349)
        fdrres = locfdr(normstat, plot=0)
        fdr.global = numeric(length(normstat))
        xx = fdrres$mat[, "x"]
        leftbreaks = xx - (xx[2]-xx[1])/2; leftbreaks[1] = min(normstat)
        rightbreaks = xx + (xx[2]-xx[1])/2; rightbreaks[length(rightbreaks)] = max(normstat)
        for (i in 1:length(normstat)) {
            ind = ((leftbreaks <= (-1)*abs(normstat[i])) | (rightbreaks >= abs(normstat[i])))
            F1l = sum(fdrres$mat[ind,"p1f1"])
            Fl = sum(fdrres$mat[ind, "f"])
            fdr.global[i] = 1 - F1l/Fl
        }

        ## go back to all genes. Genes with all 0 counts will have local and global FDR NA.
        fdr0 = rep(NA, nrow(Y))
        fdr.global0 = rep(NA, nrow(Y))
        fdr0[idx1] = fdrres$fdr
        fdr.global0[idx1] = fdr.global
        result = data.frame(result, local.fdr=fdr0, fdr=fdr.global0)
        ## fix the global FDR for first gene
        result[1,"fdr"] = result[1,"local.fdr"]
    }
    
    ## sort the results by test statistics
    ix = sort(abs(result[,"stats"] ), decreasing=TRUE, index.return=TRUE)$ix
    result = result[ix,]

    return(result)
}

locfdr <-
function(zz, bre = 120, df = 7, pct = 0, pct0 = 1/4, nulltype = 1, type = 0, plot = 1, mult, mlests, main = " ", sw = 0)
{
  call = match.call()
	if(length(bre) > 1) {
		lo <- min(bre)
		up <- max(bre)
		bre <- length(bre)
	}
	else {
		if(length(pct) > 1) {
			lo <- pct[1]
			up <- pct[2]
		}
		else {
			if(pct == 0) {
				lo <- min(zz)
				up <- max(zz)
			}
			if(pct < 0) {
				med = median(zz)
				ra = med + (1 - pct) * (range(zz) - med)
				lo = ra[1]
				up = ra[2]
			}
			if(pct > 0) {
				v <- quantile(zz, c(pct, 1 - pct))
				lo <- v[1]
				up <- v[2]
			}
		}
	}
	zzz <- pmax(pmin(zz, up), lo)
	breaks <- seq(lo, up, length = bre)
	zh <- hist(zzz, breaks = breaks, plot = F)
	x <- (breaks[-1] + breaks[ - length(breaks)])/2
	yall <- y <- zh$counts
	K <- length(y)
	N <- length(zz)
	if(pct > 0) {
		y[1.] <- min(y[1.], 1.)
		y[K] <- min(y[K], 1.)
	}
	if(type == 0) {
                X <- cbind(1, ns(x, df = df))
		f <- glm(y ~ ns(x, df = df), poisson)$fit
	}
	else {
                X <- cbind(1, poly(x, df = df))
		f <- glm(y ~ poly(x, df = df), poisson)$fit
	}
	l <- log(f)
	Fl <- cumsum(f)
	Fr <- cumsum(rev(f))
	D <- (y - f)/(f + 1)^0.5
	D <- sum(D[2:(K - 1)]^2)/(K - 2 - df)
	if(D > 1.5)
          warning(paste("f(z) misfit = ", round(
			D, 1), ".  Rerun with increased df", sep=""))
        # ............. create fp0 matrix ..........................
        if (nulltype == 3) {
                fp0 = matrix(NA, 6, 4)
                colnames(fp0) = c("delta", "sigleft", "p0", "sigright")
        }
        else {
                fp0 = matrix(NA, 6, 3)
                colnames(fp0) = c("delta", "sigma", "p0")
        }
        rownames(fp0) = c("thest", "theSD", "mlest", "mleSD", "cmest", "cmeSD")
        fp0["thest", 1:2] = c(0,1)
        fp0["theSD", 1:2] = 0
	# ..............begin central matching f0 calcs...............        
	imax <- seq(l)[l == max(l)][1]
	xmax <- x[imax]
	if(length(pct0) == 1) {
		pctup <- 1 - pct0
		pctlo <- pct0
	}
	else {
		pctlo <- pct0[1]
		pctup <- pct0[2]
	}
	lo0 <- quantile(zz, pctlo)
	hi0 <- quantile(zz, pctup)
	nx <- length(x)
	i0 <- (1.:nx)[x > lo0 & x < hi0]
	x0 <- x[i0]
	y0 <- l[i0]
	if(nulltype == 3) {
		X00 <- cbind((x0 - xmax)^2, pmax(x0 - xmax, 0)^2)
	}
	else {
		X00 <- cbind(x0 - xmax, (x0 - xmax)^2)
	}
	lr <- lm(y0 ~ X00)
	co <- lr$coef
        ## Error messages for failed CM estimation ##
        if (nulltype == 3) {
          cmerror = I(is.na(co[3]) | is.na(co[2]))
          if (!cmerror) cmerror = I(co[2] >= 0 | co[2]+co[3]>=0)
        }
        else {
          cmerror = is.na(co[3])
          if (!cmerror) cmerror = I(co[3] >= 0)
        }
        if (cmerror) {
          if (nulltype == 3)
            stop("CM estimation failed.  Rerun with nulltype = 1 or 2.")
          else
            if (nulltype == 2)
            stop("CM estimation failed.  Rerun with nulltype = 1.")
          else {
            X0 <- cbind(1, x - xmax, (x - xmax)^2)
            warning("CM estimation failed, middle of histogram non-normal")
          }
        }
        else {
	  if(nulltype == 3) {
		X0 <- cbind(1, (x - xmax)^2, pmax(x - xmax, 0)^2)
                sigs <- 1/sqrt(-2 * (c(co[2], co[2] + co[3])))
                fp0["cmest", c(1,2,4)] <- c(xmax, sigs)
	  }
	  else {
		X0 <- cbind(1, x - xmax, (x - xmax)^2)
                xmaxx <-  - co[2.]/(2. * co[3.]) + xmax
                sighat <- 1./sqrt(-2. * co[3.])
                fp0["cmest", 1:2] <- c(xmaxx, sighat)
	  }
	  l0 <- as.vector(X0 %*% co)
	  f0 <- exp(l0)
	  p0 <- sum(f0)/sum(f)
	  f0 <- f0/p0
          fp0["cmest", 3] <- p0
        }
	#............... begin MLE f0 calcs ........................
        b = 4.3 * exp(-0.26*log(N,10))
        if(missing(mlests)){
          med = median(zz);sc=diff(quantile(zz)[c(2,4)])/(2*qnorm(.75))
          mlests = locmle(zz, xlim=c(med, b*sc))
          if (N>500000) {
            warning("length(zz) > 500,000: For ML estimation, a wider interval than optimal was used.  To use the optimal interval, rerun with mlests = c(", mlests[1], ", ", b * mlests[2], ").\n", sep="")
            mlests = locmle(zz, xlim=c(med, sc))
          }
        }
	if (!is.na(mlests[1])) {
          if (N>500000) b = 1
          if (nulltype == 1) {
              Cov.in = list(x=x, X=X, f=f, sw=sw)
              ml.out = locmle(zz, xlim = c(mlests[1], b * mlests[2]),
                d=mlests[1], s=mlests[2], Cov.in=Cov.in)
              mlests = ml.out$mle
            }
            else  mlests = locmle(zz, xlim = c(mlests[1], b * mlests[2]),
                d=mlests[1], s=mlests[2])
            fp0["mlest", 1:3] = mlests[1:3]
            fp0["mleSD", 1:3] = mlests[4:6]
        }
        if (sum(is.na(fp0[c(3,5),1:2])) == 0 & nulltype > 1)
          if(abs(fp0["cmest",1] - mlests[1]) > 0.050000000000000003 |
             abs(log(fp0["cmest",2]/mlests[2])) > 0.050000000000000003)
		warning("Discrepancy between central matching and maximum likelihood estimates.\nConsider rerunning with nulltype = 1")
        ## Error messages for failed ML estimation ##
        if (is.na(mlests[1])) {
          if (nulltype == 1) {
            if (is.na(fp0["cmest", 1]))
              stop("CM and ML Estimation failed, middle of histogram non-normal")
            else stop("ML estimation failed.  Rerun with nulltype=2")
          }
          else warning("ML Estimation failed")
        }
	if(nulltype < 2) {
		delhat = xmax = xmaxx = mlests[1]
		sighat = mlests[2]
		p0 = mlests[3]
		f0 = dnorm(x, delhat, sighat)
		f0 = (sum(f) * f0)/sum(f0)
	}
        fdr = pmin((p0 * f0)/f, 1)
	f00 <- exp( - x^2/2)
	f00 <- (f00 * sum(f))/sum(f00)
	p0theo <- sum(f[i0])/sum(f00[i0])
        fp0["thest", 3] = p0theo
	fdr0 <- pmin((p0theo * f00)/f, 1)
	f0p <- p0 * f0
	if(nulltype == 0)
		f0p <- p0theo * f00
	F0l <- cumsum(f0p)
	F0r <- cumsum(rev(f0p))
	Fdrl <- F0l/Fl
	Fdrr <- rev(F0r/Fr)
	Int <- (1 - fdr) * f * (fdr < 0.90000000000000002)
	##### raise fdr to 1 near xmax .............
        if (sum(x <= xmax & fdr == 1) > 0)
          xxlo <- min(x[x <= xmax & fdr == 1])
        else xxlo = xmax
        if (sum(x >= xmax & fdr == 1) > 0)
          xxhi <- max(x[x >= xmax & fdr == 1])
        else xxhi = xmax
        if (sum(x >= xxlo & x <= xxhi) > 0)
          fdr[x >= xxlo & x <= xxhi] <- 1
        if (sum(x <= xmax & fdr0 == 1) > 0)
          xxlo <- min(x[x <= xmax & fdr0 == 1])
        else xxlo = xmax
        if (sum(x >= xmax & fdr0 == 1) > 0)
          xxhi <- max(x[x >= xmax & fdr0 == 1])
        else xxhi = xmax
	if (sum(x >= xxlo & x <= xxhi) > 0)
          fdr0[x >= xxlo & x <= xxhi] <- 1
	##################### raise fdr to 1 for mle option
	if(nulltype == 1) {
		fdr[x >= mlests[1] - mlests[2] & x <= mlests[1] + mlests[
			2]] = 1
		fdr0[x >= mlests[1] - mlests[2] & x <= mlests[1] + mlests[
			2]] = 1
	}
	p1 <- sum((1 - fdr) * f)/N
	p1theo <- sum((1 - fdr0) * f)/N
	fall <- f + (yall - y)
	########Efdr1 calculations
	Efdr <- sum((1 - fdr) * fdr * fall)/sum((1 - fdr) * fall)
	Efdrtheo <- sum((1 - fdr0) * fdr0 * fall)/sum((1 - fdr0) * fall)
	iup <- (1:K)[x >= xmax]
	ido <- (1:K)[x <= xmax]
	Eleft <- sum((1 - fdr[ido]) * fdr[ido] * fall[ido])/sum((1 - fdr[
		ido]) * fall[ido])
	Eleft0 <- sum((1 - fdr0[ido]) * fdr0[ido] * fall[ido])/sum((1 -
		fdr0[ido]) * fall[ido])
	Eright <- sum((1 - fdr[iup]) * fdr[iup] * fall[iup])/sum((1 - fdr[
		iup]) * fall[iup])
	Eright0 <- sum((1 - fdr0[iup]) * fdr0[iup] * fall[iup])/sum((
		1 - fdr0[iup]) * fall[iup])
	Efdr <- c(Efdr, Eleft, Eright, Efdrtheo, Eleft0, Eright0)
        Efdr[which(is.na(Efdr))] = 1
	names(Efdr) <- c("Efdr", "Eleft", "Eright", "Efdrtheo", "Eleft0",
		"Eright0")
	if(nulltype == 0)
		f1 <- (1 - fdr0) * fall
	else f1 <- (1 - fdr) * fall
	############ multiple sample size Efdr1 calculation
	if(!missing(mult)) {
		mul = c(1, mult)
		EE = rep(0, length(mul))
		for(m in 1:length(EE)) {
			xe = sqrt(mul[m]) * x
			f1e = approx(xe, f1, x, rule = 2, ties=mean)$y
			f1e = (f1e * sum(f1))/sum(f1e)
			f0e = f0
			p0e = p0
			if(nulltype == 0) {
				f0e = f00
				p0e = p0theo
			}
			fdre = (p0e * f0e)/(p0e * f0e + f1e)
			EE[m] = sum(f1e * fdre)/sum(f1e)
		}
		EE = EE/EE[1]
		names(EE) = mul
	}
	#................. Accuracy Calcs .................................
        Cov2.out = loccov2(X, X0, i0, f, fp0["cmest",], N)
        Cov0.out = loccov2(X, matrix(1, length(x), 1), i0, f, fp0["thest",], N)
        if(sw == 3) {
                if (nulltype==0) Ilfdr = Cov0.out$Ilfdr
                else if (nulltype==1) Ilfdr = ml.out$Ilfdr
		else if (nulltype==2) Ilfdr = Cov2.out$Ilfdr
                else stop("With sw=3, nulltype must equal 0, 1, or 2.")
	        return(Ilfdr)
              }
        if (nulltype == 0) Cov = Cov0.out$Cov
        else if (nulltype == 1) Cov = ml.out$Cov.lfdr
        else Cov = Cov2.out$Cov
	lfdrse <- diag(Cov)^0.5
        fp0["cmeSD",1:3] = Cov2.out$stdev[c(2,3,1)]
        if (nulltype==3) fp0["cmeSD",4] = fp0["cmeSD",2]
        fp0["theSD",3] = Cov0.out$stdev[1]
	########### sw==2 returns Influence function pds. ##########
	if(sw == 2) {
          if (nulltype==0) {
             pds = fp0["thest", c(3,1,2)]
             stdev = fp0["theSD", c(3,1,2)]
             pds. = t(Cov0.out$pds.)
          }
          else if (nulltype==1) {
            pds = fp0["mlest",c(3,1,2)]
            stdev = fp0["mleSD",c(3,1,2)]
            pds. = t(ml.out$pds.)
          }
          else if (nulltype==2) {
            pds = fp0["cmest",c(3,1,2)]
            stdev = fp0["cmeSD", c(3,1,2)]
            pds. = t(Cov2.out$pds.)
          }
          else stop("With sw=2, nulltype must equal 0, 1, or 2.")
          colnames(pds.) = names(pds) = c("p0", "delhat", "sighat")
          names(stdev) = c("sdp0", "sddelhat", "sdsighat")
	  return(list(pds=pds, x=x, f=f, pds.=pds., stdev=stdev))
        }
	# find cdf1, the cdf of fdr according to f1 density..................
	p1 <- seq(0.01, 0.99, 0.01)
	cdf1 <- rep(0, 99)
	fd <- fdr
	if(nulltype == 0)
		fd <- fdr0
	for(i in 1:99)
		cdf1[i] <- sum(f1[fd <= p1[i]])
        cdf1 <- cbind(p1, cdf1/cdf1[99])
	mat <- cbind(x, fdr, Fdrl, Fdrr, f, f0, f00, fdr0, yall, lfdrse,
		f1)
	namat <- c("x", "fdr", "Fdrleft", "Fdrright", "f", "f0", "f0theo",
		"fdrtheo", "counts", "lfdrse", "p1f1")
	if(nulltype == 0)
		namat[c(3, 4, 10)] <- c("Fdrltheo", "Fdrrtheo", 
			"lfdrsetheo")
	dimnames(mat) <- list(NULL, namat)
        ############## Locations of triangles ###########
        z.2 = rep(NA, 2)
        m = order(fd)[nx]
        if (fd[nx] < 0.2) {
          z.2[2] = approx(fd[m:nx], x[m:nx], 0.20000000000000001,
                               ties=mean)$y
        }
        if (fd[1] < 0.2) {
          z.2[1] = approx(fd[1:m], x[1:m], 0.20000000000000001,
                              ties=mean)$y
        }
        ################### Plotting ####################
	if(plot > 0) {
		if(plot == 2 | plot == 3)
			oldpar <- par(mfrow = c(1, 2), pty = "m")
                else if (plot ==4) oldpar = par(mfrow = c(1, 3), pty = "m")
		hist(zzz, breaks = breaks, xlab = " ", main = main)
		################### make yt positive ##############
		yt <- pmax(yall * (1 - fd), 0)
		for(k in 1:K)
		  lines(c(x[k], x[k]), c(0, yt[k]), lwd = 2, col = 6)
		if(nulltype == 3)
			title(xlab = paste("delta=", round(xmax, 3),
				"sigleft=", round(sigs[1], 3), 
				" sigright=", round(sigs[2], 3), "p0=",
				round(fp0["cmest", 3], 3)))
		if(nulltype == 1 | nulltype == 2)
			title(xlab = paste("MLE: delta:", round(
				mlests[1], 3), "sigma:", round(mlests[
				2], 3), "p0:", round(mlests[3], 3)),
                                sub = paste("CME: delta:",
                                            round(fp0["cmest",1], 3),
                                            "sigma:", round(fp0["cmest",2], 3),
                                            "p0:", round(fp0["cmest", 3], 3)))
		lines(x, f, lwd = 3, col = 3)
		if(nulltype == 0)
			lines(x, p0theo*f00, lwd = 2, lty = 2, col = 4)
		else
			lines(x, p0*f0, lwd = 2, lty = 2, col = 4)
                ################## Plot triangles ###############
                if (!is.na(z.2[2]))
		   points(z.2[2], -0.5, pch = 24, col="red", bg="yellow")
                if(!is.na(z.2[1]))
		   points(z.2[1], -0.5, pch = 24, col="red", bg="yellow")
		if(nulltype == 1 | nulltype ==2)
			Ef <- Efdr[1]
		else if (nulltype == 0) Ef <- Efdr[4]
		if (plot == 2 | plot == 4) {
			if(nulltype == 0)
				fdd <- fdr0
			else fdd = fdr
			matplot(x, cbind(fdd, Fdrl, Fdrr), type = "l",
				lwd = 3, xlab = " ", ylim = c(0, 
				1.1000000000000001), main = 
				"fdr (solid); Fdr's (dashed)")
			title(xlab = paste("Efdr= ", round(Ef, 3)))
			abline(0, 0, lty = 3, col = 2)
			lines(c(0, 0), c(0, 1), lty = 3, col = 2)
		}
		if (plot == 3 | plot == 4) {
			if(sum(is.na(cdf1[, 2])) == nrow(cdf1))
				warning("cdf1 not available")
			else {
				plot(cdf1[, 1], cdf1[, 2], type = "l",
					lwd = 3, xlab = "fdr level", ylim
					 = c(0, 1), ylab = 
					"f1 proportion < fdr level", main
					 = "f1 cdf of estimated fdr")
				title(sub = paste("Efdr= ", round(Ef,
					3)))
				lines(c(0.20000000000000001, 
					0.20000000000000001), c(0, cdf1[
					20, 2]), col = 4, lty = 2)
				lines(c(0, 0.20000000000000001), rep(
					cdf1[20, 2], 2), col = 4, lty = 2)
				text(0.050000000000000003, cdf1[20, 2],
					round(cdf1[20, 2], 2))
				abline(0, 0, col = 2)
				lines(c(0, 0), c(0, 1), col = 2)
			}
		}
		if(plot > 1)
			par(oldpar)
	}
	if(nulltype == 0) {
		ffdr <- approx(x, fdr0, zz, rule = 2, ties="ordered")$y
	}
	else ffdr <- approx(x, fdr, zz, rule = 2, ties="ordered")$y
	vl = list(fdr = ffdr, fp0 = fp0, Efdr = Efdr, cdf1 = cdf1,  mat = mat,
          z.2 = z.2)
	if(!missing(mult))
		vl$mult = EE
        vl$call = call
	vl
}

#######################################################
## some utility functions for BS-seq data
#######################################################

######################################################################
## make an object of BSseq given count data from several replicates
## The input is a list of  data frames with columns: chr, pos, N, X.
######################################################################
makeBSseqData <- function(dat, sampleNames) {
    n0 <- length(dat)

    if(missing(sampleNames))
        sampleNames <- paste("sample", 1:n0, sep="")

    alldat <- dat[[1]]
    if(any(alldat[,"N"] < alldat[,"X"], na.rm=TRUE))
        stop("Some methylation counts are greater than coverage.\n")

    ix.X <- which(colnames(alldat) == "X")
    ix.N <- which(colnames(alldat) == "N")
    colnames(alldat)[ix.X] <- "X1"
    colnames(alldat)[ix.N] <- "N1"

    if(n0 > 1) { ## multiple replicates, merge data
        for(i in 2:n0) {
            thisdat <- dat[[i]]
            if(any(thisdat[,"N"] < thisdat[,"X"], na.rm=TRUE))
                stop("Some methylation counts are greater than coverage.\n")

            ix.X <- which(colnames(thisdat) == "X")
            ix.N <- which(colnames(thisdat) == "N")
            colnames(thisdat)[c(ix.X,ix.N)] <- paste(c("X", "N"),i, sep="")
            alldat <- merge(alldat, thisdat, all=TRUE)
        }
    }

    ## make BSseq object
    ix.X <- grep("X", colnames(alldat))
    ix.N <- grep("N", colnames(alldat))
    alldat[is.na(alldat)] <- 0
    M <- as.matrix(alldat[,ix.X, drop=FALSE])
    Cov <- as.matrix(alldat[,ix.N, drop=FALSE])
    colnames(M) <- colnames(Cov) <- sampleNames

    ## order CG sites according to positions
    idx <- split(1:length(alldat$chr), alldat$chr)
    M.ordered <- M
    Cov.ordered <- Cov
    pos.ordered <- alldat$pos

    for( i in seq(along=idx) ) {
        thisidx = idx[[i]]
        thispos = alldat$pos[ thisidx ]
        dd = diff(thispos)
        if( min(dd)<0 ) { # not ordered
            warning( paste0("CG positions in chromosome ",  names(idx)[i], " is not ordered. Reorder CG sites.\n") )
            iii = order(thispos)
            M.ordered[thisidx, ] <- M[thisidx, ][iii,]
            Cov.ordered[thisidx, ] <- Cov[thisidx, ][iii,]
            pos.ordered[thisidx] <- alldat$pos[thisidx][iii]
        }
    }

    result <- BSseq(chr=alldat$chr, pos=pos.ordered, M=M.ordered, Cov=Cov.ordered)

##    result <- BSseq(chr=alldat$chr, pos=alldat$pos, M=M, Cov=Cov)

    result
}


####################################################
## naive estimates of dispersion, give X and N
####################################################
est.phi.naive <- function(X, N) {
    p = X/N
    mm = rowMeans(p)
    mm[mm==0] = 1e-5
    mm[mm==1] = 1-1e-5
    vv = rowVars(p)
    phi = vv/mm/(1-mm)
    phi[phi>=1] = 1-1e-5
    phi[phi==0] = 1e-5
    phi
}

########################################
## Merge two sets of counts.
## This is the version without smoothing
########################################
mergeData.counts <- function(BS1, BS2) {
    n1 <- getBSseq(BS1,"Cov"); x1 <- getBSseq(BS1,"M")
    n2 <-getBSseq(BS2,"Cov"); x2 <- getBSseq(BS2,"M")
    gr1 <- getBSseq(BS1,"gr");  gr2 <- getBSseq(BS2,"gr")
    allchr1 <- seqnames(gr1); allpos1 <- start(gr1)
    allchr2 <- seqnames(gr2); allpos2 <- start(gr2)

    colnames(x1) <- paste(paste("X", 1:ncol(x1), sep=""), "cond1", sep=".")
    colnames(n1) <- paste(paste("N", 1:ncol(n1), sep=""), "cond1", sep=".")
    colnames(x2) <- paste(paste("X", 1:ncol(x2), sep=""), "cond2", sep=".")
    colnames(n2) <- paste(paste("N", 1:ncol(n2), sep=""), "cond2", sep=".")

    dat1 <- data.frame(chr=as.character(seqnames(gr1)), pos=start(gr1), x1, n1)
    dat2 <- data.frame(chr=as.character(seqnames(gr2)), pos=start(gr2), x2, n2)
    alldat <- merge(dat1, dat2, all=FALSE) ## only keep the ones with data in both samples
    alldat[is.na(alldat)] <- 0

    return(alldat)
}

########################################
## Merge two sets of counts.
## This is the version with smoothing
########################################
mergeData.counts.smooth <- function(BS1, BS2, mu1.smooth, mu2.smooth) {
    n1 <- getBSseq(BS1,"Cov"); x1 <- getBSseq(BS1,"M")
    n2 <-getBSseq(BS2,"Cov"); x2 <- getBSseq(BS2,"M")
    gr1 <- getBSseq(BS1,"gr");  gr2 <- getBSseq(BS2,"gr")
    allchr1 <- seqnames(gr1); allpos1 <- start(gr1)
    allchr2 <- seqnames(gr2); allpos2 <- start(gr2)

    colnames(x1) <- paste(paste("X", 1:ncol(x1), sep=""), "cond1", sep=".")
    colnames(n1) <- paste(paste("N", 1:ncol(n1), sep=""), "cond1", sep=".")
    colnames(x2) <- paste(paste("X", 1:ncol(x2), sep=""), "cond2", sep=".")
    colnames(n2) <- paste(paste("N", 1:ncol(n2), sep=""), "cond2", sep=".")

    dat1 <- data.frame(chr=as.character(seqnames(gr1)), pos=start(gr1), x1, n1, mu1=mu1.smooth)
    dat2 <- data.frame(chr=as.character(seqnames(gr2)), pos=start(gr2), x2, n2, mu2=mu2.smooth)
    alldat <- merge(dat1, dat2, all=FALSE) ## only keep the ones with data in both samples
    alldat[is.na(alldat)] <- 0

    return(alldat)
}


######################################################################################
## function to compute means when there's no smoothing.
## Just compute the percentage of methylation.
## adding a small constant could bring trouble when there's no coverage!!!
######################################################################################
compute.mean.noSmooth <- function(X, N) {
    p <- X/N
    ##rowSums <- DelayedArray::rowSums
    const <- mean(p, na.rm=TRUE)
    p <- (rowSums(X)+const)/(rowSums(N)+1)

    nreps <-  ncol(N)
    res <- matrix(rep(p, nreps), ncol = nreps)
    return(res)
}

######################################################################################
## function to compute means when there's smoothing.
## Currently it doesn't do anything except calling smooth.collapse.
## Might add things later.
######################################################################################
compute.mean.Smooth <- function(X, N, allchr, allpos, ws=500) {
    ## rowSums <- DelayedArray::rowSums

    ## collapse the replicates and smoothing
    nreps <-  ncol(N)
    p1 <- X / N
    const <-  mean(p1, na.rm=TRUE) ##  small constant to be added
    if(nreps>1) { ## multiple replicate, collapse them. Add a small constant to bound away from 0/1
        N <- rowSums(N)+1
        X <- rowSums(X)+const
    } else {
        N <- N+0.4
        X <- X+0.4*const
    }
    ## smooth and compute p
    X.sm <- smooth.chr(as.double(X), ws, allchr, allpos)
    N.sm <- smooth.chr(as.double(N), ws, allchr, allpos)
    p <- X.sm / N.sm

    res <- matrix(rep(p, nreps), ncol = nreps)
    return(res)
}

######################################################################################
## Function to perform collapsed smoothing,
## that is, to collapse counts on replicates and then smooth
## This works for one object.
##
## 2/5/2015:
## Need a little more thinking here. How to deal with regions with no coverage???
## adding a small constant could bring trouble when there's no coverage!!!
##
## 3/2/2015: This function is not in use now. I'll remove it later.
######################################################################################
smooth.collapse <- function(BS, method, ws, ...) {
    ## grab counts and collapse
    n1 <- getBSseq(BS,"Cov")
    x1 <- getBSseq(BS,"M")
    nreps <-  ncol(n1)
    p1 <- x1 / n1
    const <-  mean(p1, na.rm=TRUE) ##  small constant to be added

    if(nreps>1) { ## multiple replicate, collapse them. Add a small constant to bound away from 0/1
        n1 <- rowSums(n1)+1
        x1 <- rowSums(x1)+const
        BS <- BSseq(chr=seqnames(BS), pos=start(BS), M=as.matrix(x1), Cov=as.matrix(n1))
    } else {
        n1 <- n1+0.4
        x1 <- x1+0.4*const
    }

    ## smooth, and then return smoothed values
    if(method == "BSmooth")  { ## use BSmooth method for smoothing. This is very slow
        res <- BSmooth(BS, h=ws, ...)
        a <- getMeth(res)[,1]
    } else { ## moving avarege. This is fast and will be default
        X.sm = smooth.chr(as.double(x1), ws, as.character(seqnames(BS)), start(BS))
        N.sm = smooth.chr(as.double(n1), ws, as.character(seqnames(BS)), start(BS))
        a = X.sm / N.sm
    }

    a
}

########################################################################
## estimate dispersion for BS-seq data, given means
########################################################################
est.dispersion.BSseq <- function(X, N, estprob, ncores) {
    prior <- est.prior.BSseq.logN(X, N)
    dispersion.shrinkage.BSseq(X, N, prior, estprob, ncores)
}

########################################################################
## A function to estimate dipersion prior for BS-seq, assuming log-normal prior.
## It takes X and N, and only use the sites with big coverages,
## then return the mean and sd of prior distribution.
##
## For single rep data: will use logN(-3,1) as prior.
########################################################################
est.prior.BSseq.logN <- function(X, N) {
    ## rowMeans = DelayedArray::rowMeans
    ## rowSums = DelayedArray::rowSums

    if(ncol(X) == 1) ## single rep
        return(c(-3, 1))

    ## keep sites with large coverage and no missing data
    ix = rowMeans(N>10)==1 & rowSums(N==0)==0
    if(sum(ix) < 50) {
        warning("The coverages are too low. Cannot get good estimations of prior. Use arbitrary prior N(-3,1).")
        return(c(-3, 1))
    }

    X = X[ix,,drop=FALSE]
    N = N[ix,,drop=FALSE]
    ## compute sample mean/var
    p = X/N
    mm = rowMeans(p)
    mm[mm==0] = 1e-5
    mm[mm==1] = 1-1e-5
    vv = rowVars(p)
    phi = vv/mm/(1-mm)
    ## exclude those with vv==0. Those are sites with unobservable phis.
    ## But this will over estimate the prior.
    ## What will be the consequences????
    phi = phi[vv>0]
    lphi = log(phi[phi>0])
    prior.mean = median(lphi, na.rm=TRUE)
    prior.sd = IQR(lphi, na.rm=TRUE) /1.39

    ## It seems this over-estimates the truth. Need to use the tricks in
    ## my biostat paper to remove the over-estimation. To be done later.
    c(prior.mean, prior.sd)
}

########################################################################
## Dispersion shrinkage based on log-normal penalized likelihood.
## Takes X, N, estimated mean and prior.
##
## The shrinakge is done in log scale. So data will be shrink to the
## logarithmic means.
########################################################################
dispersion.shrinkage.BSseq <- function(X, N, prior, estprob, ncores) {
    ## penalized likelihood function
    plik.logN <- function(size, X,mu,m0,tau,phi)
        -(sum(dbb(size, X, mu, exp(phi))) + dnorm(phi, mean=m0, sd=tau, log=TRUE))

    ## for CG sites with no coverage, use prior
    shrk.phi=exp(rep(prior[1],nrow(N)))

    ## deal with estprob, make it a matrix if not.
    if(!is.matrix(estprob))
        estprob <- as.matrix(estprob)

    ## skip those without coverage
    ix <- rowSums(N>0) > 0
    X2 <- X[ix, ,drop=FALSE]; N2 <- N[ix,,drop=FALSE]; estprob2 <- estprob[ix,,drop=FALSE]
    shrk.phi2 <- rep(0, nrow(X2))

    ## setup a progress bar
    nCG.pb = round(nrow(X2)/100)
    if(ncores == 1) { ## use single core, no parallelism
        pb <- txtProgressBar(style = 3)
        for(i in 1:nrow(X2)) {
            ## print a progress bar
            if((i %% nCG.pb) == 0)
                setTxtProgressBar(pb, i/nrow(X2))
            ## I can keep the 0's with calculation. They don't make any difference.
            shrk.one=optimize(f=plik.logN, size=N2[i,], X=X2[i,], mu=estprob2[i,], m0=prior[1], tau=prior[2],
            interval=c(-5, log(0.99)),tol=1e-3)
            shrk.phi2[i]=exp(shrk.one$minimum)
        }
        setTxtProgressBar(pb, 1)
        cat("\n")
    } else  { ## use multiple cores.
        foo <- function(i) {
            shrk.one <- optimize(f=plik.logN, size=N2[i,], X=X2[i,],
                                 mu=estprob2[i,], m0=prior[1], tau=prior[2],
                                 interval=c(-5, log(0.99)),tol=1e-3)
            exp(shrk.one$minimum)
        }
        shrk.phi2 <- try(mclapply(1:nrow(X2), foo, mc.cores=ncores))
        if(class(shrk.phi2) == "try-error") {
            stop(paste("Shrinkage estimator in parallel computing encountered errors.",
                       "Please switch to single core.\n"))
        } else {
            shrk.phi2 <- unlist(shrk.phi2)
        }
    }

##         shrk.phi2 = foreach ( i=1:nrow(X2), .combine=c )  %dopar% {
##             ## print a progress bar
##             if((i %% nCG.pb) == 0)
##                 setTxtProgressBar(pb, i/nrow(X2))
##             ## I can keep the 0's with calculation. They don't make any difference.
##             shrk.one = optimize(f=plik.logN, size=N2[i,], X=X2[i,], mu=estprob2[i,], m0=prior[1], tau=prior[2],
##             interval = c(-5, log(0.99)),tol=1e-3)
##             exp(shrk.one$minimum)
##         }
##         stopImplicitCluster()
##         setTxtProgressBar(pb, 1)
##         cat("\n")

    shrk.phi[ix] <- shrk.phi2

    return(shrk.phi)
}

#########################################################
## beta-binomial (BB) density function.
## The BB distribution is parametrized by mean and dispersion.
#########################################################
dbb <- function (size, x, mu, phi, log=TRUE)  {
    ## 'size' and/or 'x' could be DelayedArray objects so turn them into
    ## ordinary arrays
    size=as.array(size)
    x=as.array(x)
    ## first convert mu/phi to alpha/beta
    tmp=1/phi-1
    alpha=mu*tmp
    beta=tmp - alpha
    v=lchoose(size,x)-lbeta(beta, alpha)+lbeta(size-x + beta,x+alpha)
    if(!log)
        return(exp(v))
    else return(v)
}

###########################################
## sort according to chr and pos.
## Return the index
###########################################
sortPos <- function(chr, pos) {
    do.call(order, list(chr, pos))
}

---
title: "The DSS User's Guide"
shorttitle: "DSS guide"
author:
  - Hao Wu, hao.wu@emory.edu
package: DSS
bibliography: DSS.bib
abstract: >
  This vignette introduces the use of the Bioconductor package
  DSS (Dispersion  Shrinkage for Sequencing data), which is designed for
  differential analysis on high-throughput sequencing data.
  It provide functionalities for RNA-seq differential expression,
  and bisulfite sequencing (BS-seq) differential methylation.
  The core of DSS is a procedure based on Bayesian hierarchical model
  to estimate and shrink gene- or CpG site-specific dispersions,
  then conduct Wald tests for detecting differential expression/methylation.

vignette: >
  %\VignetteIndexEntry{The DSS User's Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    toc_float: true
---




# Introduction

## Background

Recent advances in various high-throughput sequencing technologies have
revolutionized genomics research. Among them,
RNA-seq is designed to measure the the abundance of RNA products,
and Bisulfite sequencing (BS-seq) is for measuring DNA methylation.
A fundamental question in functional genomics research
is whether gene expression or DNA methylation
vary under different biological contexts.
Thus, identifying differential expression genes (DEGs)
or differential methylation loci/regions (DML/DMRs)
are key tasks in RNA-seq or BS-seq data analyses.

The differential expression (DE) or differential methylation (DM) analyses
are often based on gene- or CpG-specific statistical test.
A key limitation in RNA- or BS-seq experiments
is that the number of biological replicates is usually limited due to cost constraints.
This can lead to unstable estimation of within group variance,
and subsequently undesirable results from hypothesis testing.
Variance shrinkage methods have been widely applied in DE analyses in
microarray data to improve the estimation of gene-specific within group variances.
These methods are typically based on a Bayesian hierarchical model,
with a prior imposed on the gene-specific variances to provide
a basis for information sharing across all genes.

A distinct feature of RNA-seq or BS-seq data is that the measurements are
in the form of counts and have to be modeld by discrete distributions.
Unlike continuous distributions such as Gaussian,
the variances depend on means in these distributions.
This implies that the sample variances do not account for biological variation,
and shrinkage cannot be applied on variances directly.
In DSS, we assume that the count data are
from the negative-binomial (for RNA-seq) or beta-binomial (for BS-seq) distribution.
We then parameterize the distributions by a mean and a dispersion parameters.
The dispersion parameters, which represent the biological variation for
replicates within a treatment group, play a central role in the differential analyses.

DSS implements a series of DE/DM detection algorithms based on the
dispersion shrinkage method followed by Wald statistical test
to test each gene/CpG site for differential expression/methylation.
It provides functions for RNA-seq DE analysis for both two group comparision and multi-factor design,
BS-seq DM analysis for two group comparision, multi-factor design, and data without biological replicate.

For more details of the data model, the shrinkage method, and test procedures,
please read [@DE] for differential expression from RNA-seq,
[@DML] for differential methylation for two-group comparison from BS-seq,
[@DMR1] for differential methylation for data without biological replicate,
and [@DML-general] for differential methylation for general experimental design.

## Citation

* For differential expression in RNA-seq, cite @DE. 
* For general differential methylation in BS-seq, cite @DML. 
* For differential methylation in BS-seq when there's no biological replicate, cite @DMR1.
* For differential methylation in BS-seq under general experimental design, cite @DML-general.


# Using DSS for RNA-seq differential expression analysis

## Input data preparation

DSS requires a count table (a matrix of integers) for gene expression values
(rows are for genes and columns are for samples).
This is different from the isoform expression based analysis such as in cufflink/cuffdiff,
where the gene expressions are represented as non-integers values.

There are a number of ways to obtain the count table from raw sequencing data (fastq file).
Aligners such as [STAR](https://github.com/alexdobin/STAR) automatically output a count table. 
Several Bioconductor packages serve this purpose, 
for example, [Rsubread](http://bioconductor.org/packages/release/bioc/html/Rsubread.html), 
[QuasR](http://bioconductor.org/packages/release/bioc/html/QuasR.html), 
and [easyRNASeq](http://bioconductor.org/packages/release/bioc/html/easyRNASeq.html). 
Other tools like 
[Salmon](http://combine-lab.github.io/salmon/), 
[Kallisto](https://pachterlab.github.io/kallisto/about.html), 
or [RSEM](http://deweylab.github.io/RSEM/) can also be used. 
Please refer to the package manuals for more details.

## Single factor experiment

In single factor RNA-seq experiment, DSS requires
a vector representing experimental designs. The length of the
design vector must match the number of columns of the count table.
Optionally, normalization factors or additional annotation for genes can be supplied.

The basic data container in the package is ``SeqCountSet`` class,
which is directly inherited from ``ExpressionSet`` class
defined in ``Biobase``. An object of the class contains all necessary
information for a DE analysis: gene expression values, experimental designs,
and additional annotations.

A typical DE analysis contains the following simple steps.
- Create a  ``SeqCountSet`` object using ``newSeqCountSet``.
- Estimate normalization factor using ``estNormFactors``.
- Estimate and shrink gene-wise dispersion using ``estDispersion``.
- Two-group comparison using ``waldTest``.

The usage of DSS is demonstrated in the simple simulation below.

- First load in the library, and make a ``SeqCountSet``
object from simulated counts for 2000 genes and 6 samples.

```{r,message=FALSE}
library(DSS)
counts1 = matrix(rnbinom(300, mu=10, size=10), ncol=3)
counts2 = matrix(rnbinom(300, mu=50, size=10), ncol=3)
X1 = cbind(counts1, counts2)
X2 = matrix(rnbinom(11400, mu=10, size=10), ncol=6)
X = rbind(X1,X2)  ## these are 100 DE genes
designs = c(0,0,0,1,1,1)
seqData = newSeqCountSet(X, designs)
seqData
```

- Estimate normalization factor.
```{r}
seqData = estNormFactors(seqData)
```

- Estimate and shrink gene-wise dispersions
```{r}
seqData = estDispersion(seqData)
```

- With the normalization factors and dispersions ready, the two-group comparison can be
conducted via a Wald test:
```{r}
result=waldTest(seqData, 0, 1)
head(result,5)
```

A higher level wrapper function  ``DSS.DE`` is provided
for simple RNA-seq DE analysis in a two-group comparison.
User only needs to provide a count matrix and a vector of 0's and 1's representing the
design, and get DE test results in one line. A simple example is listed below:
```{r}
counts = matrix(rpois(600, 10), ncol=6)
designs = c(0,0,0,1,1,1)
result = DSS.DE(counts, designs)
head(result)
```


## Multifactor experiment

``DSS`` provides functionalities for dispersion shrinkage for multifactor experimental designs.
Downstream model fitting (through genearlized linear model)
and hypothesis testing can be performed using other packages such as ``edgeR``,
with the dispersions estimated from DSS.

Below is an example, based a simple simulation, to illustrate the DE analysis of
a crossed design.

- First simulate data for a 2x2 crossed experiments. Note the
counts are randomly generated.
```{r, message=FALSE}
library(DSS)
library(edgeR)
counts = matrix(rpois(800, 10), ncol=8)
design = data.frame(gender=c(rep("M",4), rep("F",4)), 
                    strain=rep(c("WT", "Mutant"),4))
X = model.matrix(~gender+strain, data=design)
```

- make SeqCountSet, then estimate size factors and  dispersion
```{r}
seqData = newSeqCountSet(counts, as.data.frame(X))
seqData = estNormFactors(seqData)
seqData = estDispersion(seqData)
```


- Using edgeR's function to do glm model fitting, but plugging in the estimated dispersion
from DSS.
```{r}
fit.edgeR = glmFit(counts, X, dispersion=dispersion(seqData))
```

- Using edgeR's function to do hypothesis testing on the second parameter of the model (gender).
```{r}
lrt.edgeR = glmLRT(glmfit=fit.edgeR, coef=2)
head(lrt.edgeR$table)
```


# Using DSS for BS-seq differential methylation analysis

## Overview

To detect differential methylation, statistical tests are conducted at each CpG site,
and then the differential methylation loci (DML) or differential methylation regions (DMR)
are called based on user specified threshold.
A rigorous statistical tests should account for
biological variations among replicates and the sequencing depth.
Most existing methods for DM analysis are based on  ad hoc methods.
For example, using Fisher's exact ignores the biological variations,
using t-test on estimated methylation levels ignores the sequencing depth.
Sometimes arbitrary filtering are implemented: loci with  depth
lower than an arbitrary threshold are filtered out, which results in information loss

The DM detection procedure implemented in DSS is based on
a rigorous Wald test for beta-binomial distributions.
The test statistics depend on the biological variations (characterized by dispersion parameter)
as well as the sequencing depth. An important part of the algorithm
is the estimation of dispersion parameter, which is achieved through a
shrinkage estimator based on a Bayesian hierarchical model [@DML].
An advantage of DSS is that the test can be performed even when
there is no biological replicates. That's because by smoothing,
the neighboring CpG sites can be viewed as ***pseudo-replicates***, and the dispersion
can still be estimated with reasonable precision.

DSS also works for general experimental design,  based on a beta-binomial
regression model with ***arcsine*** link function.
Model fitting is performed on transformed data with generalized least
square method, which achieves much improved computational performance compared
with methods based on generalized linear model.

DSS depends on [bsseq](http://bioconductor.org/packages/release/bioc/html/bsseq.html) 
Bioconductor package, which has neat definition of
data structures and many useful utility functions. In order to use the DM detection functionalities,
``bsseq`` needs to be pre-installed.



## Input data preparation

DSS requires data from each BS-seq experiment to be summarized
into following information for each CG position:
chromosome number, genomic coordinate, total number of reads,
and number of reads showing methylation. For a sample, this information
are saved in a simple text file, with each row representing a CpG site.
Below shows an example of a small part of such a file:

```
chr     pos     N       X
chr18   3014904 26      2
chr18   3031032 33      12
chr18   3031044 33      13
chr18   3031065 48      24
```

One can follow below steps to obtain such data from raw sequence file (fastq file),
using [bismark](https://www.bioinformatics.babraham.ac.uk/projects/bismark/) 
(version 0.10.0, commands for newer versions could be different)
for BS-seq alignment and count extraction. These steps require installation
of [bowtie](http://bowtie-bio.sourceforge.net/index.shtml) 
or [bowtie2](http://bowtie-bio.sourceforge.net/bowtie2/index.shtml), 
``bismark``, and the fasta file for reference genome.

1. Prepare Bisulfite reference genome. This can be done using the
``bismark_genome_preparation`` function (details in bismark manual). Example command is:
```
bismark_genome_preparation --path_to_bowtie /usr/local/bowtie/ \
  --verbose /path/to/refgenomes/
```

2. BS-seq alignment. Example command is:
```
bismark -q -n 1 -l 50  --path_to_bowtie \
  /path/bowtie/ BS-refGenome reads.fastq
```
This step will produce two text files ``reads.fastq_bismark.sam`` and
``reads.fastq_bismark_SE_report.txt``.

3. Extract methylation counts using ``bismark_methylation_extractor`` function: 
```
bismark_methylation_extractor -s --bedGraph reads.fastq_bismark.sam
```
This will create multiple txt files to summarize methylation call and cytosine context,
a bedGraph file to display methylation percentage, and a coverage file containing counts information.
The count file contain following columns: ``chr, start, end, methylation%, count methylated, count unmethylated``.
This file can be modified to make the input file for DSS.


A typical DML detection contains two simple steps. First one conduct
DM test at each CpG site, then DML/DMR are called based on the test result
and user specified threshold.



## DML/DMR detection from two-group comparison

**Step 1**. Load in library. Read in text files and create an object of ``BSseq`` class, which is
defined in ``bsseq`` Bioconductor package.

```{r}
library(DSS)
require(bsseq)
path = file.path(system.file(package="DSS"), "extdata")
dat1.1 = read.table(file.path(path, "cond1_1.txt"), header=TRUE)
dat1.2 = read.table(file.path(path, "cond1_2.txt"), header=TRUE)
dat2.1 = read.table(file.path(path, "cond2_1.txt"), header=TRUE)
dat2.2 = read.table(file.path(path, "cond2_2.txt"), header=TRUE)
BSobj = makeBSseqData( list(dat1.1, dat1.2, dat2.1, dat2.2),
     c("C1","C2", "N1", "N2") )[1:1000,]
BSobj
```


**Step 2**. Perform statistical test for DML by calling ``DMLtest`` function.
This function basically performs following steps: (1) estimate mean methylation levels
for all CpG site; (2) estimate dispersions at each CpG sites; (3) conduct Wald test.
For the first step, there's an option for smoothing or not.  Because the methylation levels
show strong spatial correlations, smoothing can often help obtain better estimates
of mean methylation.
<!-- when the CpG sites are dense in the data
(such as from the whole-genome BS-seq). However for data with sparse CpG,
such as from RRBS or hydroxymethylation, smoothing is not recommended. -->


To perform DML test without smoothing, do:
```{r, results=FALSE, eval=FALSE}
dmlTest = DMLtest(BSobj, group1=c("C1", "C2"), group2=c("N1", "N2"))
```

To perform statistical test for DML with smoothing, do:
```{r, results=FALSE, eval=FALSE}
dmlTest.sm = DMLtest(BSobj, group1=c("C1", "C2"), group2=c("N1", "N2"), 
                     smoothing=TRUE)
```

User has the option to smooth the methylation levels or not. 
A simple moving average algorithm is implemented for smoothing.
For WGBS data, smoothing is always recommended
so that information from nearby CpG sites can be combined to improve the estimation of methylation levels.
For data with sparse CpG coverage such as RRBS or hydroxymethylation, 
smoothing might not alter the results much, but is still recommended. 
In RRBS, CpG sites are likely to be clustered locally within small genomic regions, 
so smoothing can potentially help the methylation estimation.

If smoothing is requested, smoothing span is an important parameter which has non-trivial
impact on DMR calling. We use 500 bp as default, because it performs well in real data tests
according to our experience.


**Step 3**. With the test results, one can call DML by using ``callDML`` function.
The results DMLs are sorted by the significance.
```{r, echo=FALSE, result=FALSE, include=FALSE, prompt=FALSE}
  dmlTest = DMLtest(BSobj, group1=c("C1", "C2"), group2=c("N1", "N2"),
                    ncores=1)
```

```{r}
  dmls = callDML(dmlTest, p.threshold=0.001)
  head(dmls)
```

By default, the test is based on the null hypothesis that the difference in methylation levels is 0.
Alternatively, users can specify a threshold for difference. For example, to
detect loci with difference greater than 0.1, do:
```{r}
  dmls2 = callDML(dmlTest, delta=0.1, p.threshold=0.001)
  head(dmls2)
```

When delta is specified, the function will compute the posterior probability that the
difference of the means is greater than delta. So technically speaking,
the threshold for p-value here actually refers to the threshold for 1-posterior probability,
or the local FDR. Here we use the same parameter name for the sake of
the consistence of function syntax.

**Step 4**. DMR detection is also Based on the DML test results, by calling ``callDMR`` function.
Regions with many statistically significant CpG sites are identified as DMRs.
Some restrictions are provided by users, including the minimum
length, minimum number of CpG sites, percentage of CpG site being significant
in the region, etc. There are some  post hoc procedures to merge nearby DMRs into longer ones.

```{r}
dmrs = callDMR(dmlTest, p.threshold=0.01)
head(dmrs)
```

Here the DMRs are sorted by ``areaStat``, which is defined in ``bsseq``
as the sum of the test statistics of all CpG sites within the DMR.

Similarly, users can specify a threshold for difference. For example, to
detect regions with difference greater than 0.1, do:

```{r}
  dmrs2 = callDMR(dmlTest, delta=0.1, p.threshold=0.05)
  head(dmrs2)
```

Note that the distribution of test statistics (and p-values) depends on
the differences in methylation levels and biological variations,
as well as technical factors such as coverage depth. It is very difficulty
to select a natural and rigorous threshold for defining DMRs. We recommend
users try different thresholds in order to obtain satisfactory results.

The DMRs can be visualized using ``showOneDMR`` function,
This function provides more information than the ``plotRegion`` function in ``bsseq``.
It plots the methylation percentages as well as the coverage depths
at each CpG sites, instead of just the smoothed curve.
So the coverage depth information will be available in the figure.

To use the function, do
```{r, eval=FALSE}
  showOneDMR(dmrs[1,], BSobj)
```

The result figure looks like the following.
**Note that the figure below is not generated from the above example.
The example data are from RRBS experiment so the DMRs are much shorter.**

![](aDMR.png)

### Parallel computing for DML/DMR detection from two-group comparison

We implement parallel computing for two-group DML test to speed up the computation. 
The parallelism is achieved by specifying the `ncores` parameter in `DMLtest` function,  
through the functionalities provided in  `parallel` package. 
Please see the help for `DMLtest` function for more description and example codes. 
Note that older version of DSS (<2.4x) used the `bplapply` function in `BiocParallel`
package. However, that function somehow has significantly reduced
performance in the newer release (>1.25), so we switched to `mcapply`.
A drawback is that the progress bar cannot be displayed under the paralelle computing setting.
Another problem is that users might experience problems using parallel computing on Windows,
since the mcapply function relies on forking but Windows does not support forking.
Thus, we suggest to use ncores=1 on Windows. For more details,
please read the `parallel` package documentation.

We did some simple tests. The figure below shows the time spent in `DMLtest` function,
for different numbers of CpG sites in the data and using different number of cores. 
It shows significant improvements using multiple cores. 

![](parallel.png)

## DML/DMR detection from general experimental design

In DSS, BS-seq data from a general experimental design (such as crossed experiment,
or experiment with covariates) is modeled through a generalized linear model framework.
We use **arcsine** link function instead of the typical logit link for it better deals with data
at boundaries (methylation levels close to 0 or 1).  Linear model fitting is done
through ordinary least square on transformed methylation levels.
Variance/covariance matrices for the estimates are derived with consideration
of count data distribution and transformation.


### Hypothesis testing in general experimental design

In a general design, the data are modeled through a multiple regression framework,
thus there are several regression coefficients. In contrast, there is only one parameter
in two-group comparison which is the difference between two groups.
Under this type of design, hypothesis testing can be performed for one, multiple,
or any linear combination of the parameters.

DSS provides flexible functionalities for hypothesis testing.
User can test one parameter in the model through a Wald test,
or any linear combination of the parameters through an F-test.

The ``DMLtest.multiFactor`` function provide interfaces for testing one parameter
(through ``coef`` parameter), one term in the model (through ``term`` parameter),
or linear combinations of the parameters (through ``Contrast`` parameter).
We illustrate the usage of these parameters through a simple example below.
Assume we have an experiment from three strains (A, B, C) and two sexes (M and F),
each has 2 biological replicates (so there are 12 datasets in total).

```{r}
Strain = rep(c("A", "B", "C"), 4)
Sex = rep(c("M", "F"), each=6)
design = data.frame(Strain,Sex)
design
```

To test the additive effect of Strain and Sex, a design formula is ``~Strain+Sex``,
and the corresponding design matrix for the linear model is:
```{r}
X = model.matrix(~Strain+ Sex, design)
X
```

Under this design, we can do different tests using the ``DMLtest.multiFactor`` function:

- If we want to test the sex effect, we can either specify ``coef=4`` 
(because the 4th column in the design matrix corresponds to sex), 
``coef="SexM"``, or ``term="Sex"``. It is important to note that when using character for coef, 
the character must match the column name of the
design matrix, i.e., one cannot do ``coef="Sex"``. It is also important to
note that using ``term="Sex"`` only tests a single paramter in the model
because sex only has two levels.

- If we want to test the effect of Strain B versus Strain A (this is also testing a single parameter),
we do ``coef=2`` or ``coef="StrainB"``.

- If we want to test the whole Strain effect, it becomes a compound test because Strain has three levels.
We do ``term="Strain"``, which tests ``StrainB`` and ``StrainC`` simultaneously.
We can also make a Contrast matrix L as following. It's clear that testing $L^T \beta = 0$ is equivalent
to testing StrainB=0 and StrainC=0.
```{r}
L = cbind(c(0,1,0,0),c(0,0,1,0))
L
```

- One can perform more general test, for example, to test StrainB=StrainC,
or that strains B and C has no difference (but they could be different from Strain A).
In this case, we need to make following contrast matrix:
```{r}
matrix(c(0,1,-1,0), ncol=1)
```


### Example analysis for data from general experimental design

**Step 1**. Load in data distributed with ``DSS``. This is a small portion of a set of
RRBS experiments. There are 5000 CpG sites and 16 samples.
The experiment is a $2\times2$ design (2 cases and 2 cell types).
There are 4 replicates in each case-by-cell combination.

```{r}
data(RRBS)
RRBS
design
```

**Stepp 2**. Fit a linear model using ``DMLfit.multiFactor`` function, include
case, cell, and case by cell interaction. Similar to in a multiple regression, 
the model only needs to be fit once, and then the parameters can be tested 
based on the model fitting results.

```{r}
DMLfit = DMLfit.multiFactor(RRBS, design=design, formula=~case+cell+case:cell)
```

**Step 3**. Use ``DMLtest.multiFactor`` function to test the cell effect.
It is important to note that the ``coef`` parameter is the index
of the coefficient to be tested for being 0. Because the model
(as specified by ``formula`` in ``DMLfit.multiFactor``) include intercept,
the cell effect is the 3rd column in the design matrix, so we use
``coef=3`` here.

```{r}
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef=3)
```

Alternatively, one can specify the name of the parameter to be tested.
In this case, the input ``coef`` is a character, and it must match one of
the column names in the design matrix. The column names of the design matrix
can be viewed by
```{r}
colnames(DMLfit$X)
```

The following line also works. Specifying ``coef="cellrN"`` is the same as
specifying {\tt coef=3}.

```{r}
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef="cellrN")
```

Result from this step is a data frame with chromosome number, CpG site position,
test statistics, p-values (from normal distribution), and FDR.
Rows are sorted by chromosome/position of the CpG sites.
To obtain top ranked CpG sites, one can sort the data frame using following codes:

```{r}
ix=sort(DMLtest.cell[,"pvals"], index.return=TRUE)$ix
head(DMLtest.cell[ix,])
```

<!-- Below is a figure showing the distributions of test statistics and p-values from this example dataset -->

<!-- ```{r} -->
<!-- par(mfrow=c(1,2)) -->
<!-- hist(DMLtest.cell$stat, 50, main="test statistics", xlab="") -->
<!-- hist(DMLtest.cell$pvals, 50, main="P values", xlab="") -->
<!-- ``` -->

**Step 4*. DMRs for multifactor design can be called using {\tt callDMR} function:
```{r}
callDMR(DMLtest.cell, p.threshold=0.05)
```

Note that for results from for multifactor design, {\tt delta} is NOT supported.
This is because in multifactor design, the estimated
coefficients in the regression are based on a GLM framework (loosely
speaking), thus they don't have clear meaning of methylation level
differences. So when the input DMLresult is from {\tt DMLtest.multiFactor},
{\tt delta} cannot be specified.

### More flexible way to construct a hypothesis test

 Following 4 tests should produce the same results,  since 'case' only has two levels.
 However the p-values from F-tests (using term or Contrast) are
 slightly different, due to normal approximation in Wald test.
 
```{r}
## fit a model with additive effect only
DMLfit = DMLfit.multiFactor(RRBS, design, ~case+cell)
## test case effect
test1 = DMLtest.multiFactor(DMLfit, coef=2)
test2 = DMLtest.multiFactor(DMLfit, coef="caseSLE")
test3 = DMLtest.multiFactor(DMLfit, term="case")
Contrast = matrix(c(0,1,0), ncol=1)
test4 = DMLtest.multiFactor(DMLfit, Contrast=Contrast)
cor(cbind(test1$pval, test2$pval, test3$pval, test4$pval))
```

The model fitting and hypothesis test procedures are computationally very efficient.
For a typical RRBS dataset with 4 million CpG sites,
it usually takes less than half hour. In comparison, other similar software such as RADMeth or BiSeq
takes at least 10 times longer.

### For paired design
DSS can handle paired design using a **fixed effect model**. 
To be specific, assuming you have a design with 3 control and 3 treated samples,
and the control and treated samples are paired. 
The design data frame will be constructed as the following.
**It's important to make the pair information as factor**. 

```{r}
Treatment = factor(rep(c("Control","Treated"), 3))
pair = factor( rep(1:3, each=2) )
design = data.frame(Treatment, pair)
design
```

To fit the model, "pair" variable needs to be included in the formula.
Then the  treatment effect can be tested adjusted for pairing.
Note, the following is for illustration purpose only since the data
are not really paired. The input
`BSobj` needs to match the dimension of `design`, i.e.,
the number of samples in `BSobj` should equal to the number of
rows of `design`.

```{r, eval=FALSE}
BSobj=RRBS[,1:6]
DMLfit = DMLfit.multiFactor(BSobj, design, formula = ~ Treatment + pair)
dmlTest = DMLtest.multiFactor(DMLfit, term="Treatment")
```

Moreover, datasets with repeated measurements (such as longituinal data)
can be analyzed using similar fashion under fixed effect model. 
We acknowledge that mixed effect models are potentially more efficient, 
but we have not implemented mixed model in DSS. 


# Frequently Asked Questions

## For BS-seq data analysis

**Q: Do I need to filter out CpG sites with low or no coverage?**

A: No. DSS will take care of the coverage depth information. 
The depth will be factored into the statistical computation. 

**Q: Can I use estimated methylation levels (beta values) with DSS?**

A: No. DSS only takes count data. I don't recommend compute methylation levels out of the counts
and then use the point estimates as input, since that will complete ignore the 
sequencing depth information. For example, 1 out 2 and 100 out of 200 both give 50% methylation level,
but the precision of the point estimates are very different. 

**Q: Should I always do smoothing, even for RRBS data?**

A: Smoothing is always recommended, even for RRBS. In RRBS, CpG sites are likely to be clustered 
locally within small genomic regions, so smoothing will still help to improve the methylation estimation. 

**Q: Why doesn't ``DMLtest.multiFactor`` function return the relative methylation levels?**

A: There are several reasons. First, in functions for general design, we use arcsin 
link function with a multiple regression setting, thus the estimated regression coefficients
are not relative methylation levels. Returning these values will not be very meaningful.
Secondly, the relative methylation levels in a multiple regression setting is a
quantity to difficult to comprehend for user without formal statistical training.  
It is similar to a multiple regression, where the interpretation of the 
coefficient is something like "the change of Y with 1 unit increase of X, 
adjusting for other covariates". Considering X can be anything (even continuous), 
we don't feel there is a clear definition of relative methylation level,
unlike in two-group comparison setting. 
Due to these reasons, we decided not to compute and  return those values. 

**Q: Why doesn't callDMR function return FDR  for identified DMRs?**

A:  In DMR calling, the statistical test is perform for each CpG, and then the significant CpG are 
merged into regions. It is very difficult to estimate FDR at the 
region-level, based on the site level test results (p-values or FDR). 
This is a difficult question for all types of genome-wide assays such as the ChIP-seq. 
So I rather not to report a DMR-level FDR if it’s not accurate.

**Q: What's the meaning of areaStat parameter?**

A: We adapt that from the ``bsseq`` package. It is the sum of the 
test statistics of all CpG sites within a DMR. 
It doesn’t have a direct biological meaning. One can imagine when we try to rank the DMRs, 
we don't know whether the height or the width is more important?
AreaStat is a combination of the two. This is an ad hoc way to rank DMRs, 
but larger AreaStat is more likely to be a DMR

**Q: How can I get genome annotation (like the nearby genes) for the DMRs?**

A: DSS doesn't provide this functionality. There are many other tools can do this, 
for example, HOMER. 


**Q: What's the memory requirement for DSS?**

A: It is really difficult to tell. According to my experience, 
a whole genome BS-seq dataset with 20+ million CpG sites and 4 samples 
can run on my laptop with 16G RAM. I once used DelayedArray in DSS
to reduce the memory usage. But it makes the computation significantly slower so I changed back. 

**Q: How to consider the batch effect with DSS?**

A: I think the best strategy will be using the multifactor functions
with batch as a covariate in the model. 


**Q: Does DSS work for survival data?**

A. No. But these are on our future development plan. 


# Session Info

```{r}
sessionInfo()
```

# References

\name{design}
\alias{design}
\docType{data}
\title{Experimental design for the example RRBS dataset}

\description{
  The RRBS dataset is from 16 samples with two factors (case and cell),
  each has two levels (so it's a 2x2 design). 
}
\usage{data(RRBS)}
\examples{
data(RRBS)
RRBS
design
}
\keyword{datasets}

\name{DSS-package}
\docType{package}
\alias{DSS}
\alias{DSS-package}
\title{
  Dispersion shrinkage for sequencing data
}
\description{
  DSS is an R library performing the differential expression analysis for
  RNA-seq count data. Compared with other similar packages (DESeq,
  edgeR), DSS implements a new dispersion shrinkage method to estimate the
  gene-specific biological variance. Extensive simulation results
  showed that DSS performs favorabily compared to DESeq and edgeR when
  the variation of biological variances is large.

  DSS only works for two group comparison at this time. We plan to
  extend the functionalities and make it work for more general
  experimental designs in the near future. 
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\keyword{package}

\name{DMLfit.multiFactor}
\alias{DMLfit.multiFactor}

\title{
  Fit a linear model for BS-seq data from general experimental design
}

\description{
  This function takes a BSseq object, a data frame for experimental
  design and a model formula and then fit a linear model. 
}

\usage{
DMLfit.multiFactor(BSobj, design, formula, smoothing=FALSE, smoothing.span=500)
}

\arguments{
  \item{BSobj}{An object of BSseq class for the BS-seq data.}
  \item{design}{A data frame for experimental design. Number of rows
	must match the number of columns of the counts in BSobj.}
  \item{formula}{A formula for the linear model.}
  \item{smoothing}{A flag to indicate whether to apply smoothing. When
	true, the counts will be smoothed by a simple moving average method.}
  \item{smoothing.span}{The size of smoothing window, in
	basepairs. Default is 500.} 
}

\details{
  The lineear model fitting is done through ordinary least square on the
  arscine transformed methylation percentages. The estimated standard
  errors are computed with consideration of the data (count)
  distribution and transformation. This function is extremely
  efficient. The computation takes around 20 minutes for 4 million CpG
  sites.
}

\value{
  A list with following components
  \item{gr}{An object of 'GRanges' for locations of the CpG sites.}
  \item{design}{The input data frame for experimental design.}
  \item{formula}{The input formula for the model.}
  \item{X}{The design matrix used in regression. It is created based on
  design and formula.}
  \item{fit}{The model fitting results. This is a list itself, with three
  components: 'beta' - the estimated coefficients; 'var.beta' - estimated
  variance/covariance matrices for beta. 'phi' - estimated beta-binomial
  dispersion parameters.  Note that var.beta for a CpG
  site should be a ncol(X) x ncol(X) matrix, but is flattend to a vector
  so that the matrices for all CpG sites can be saved as a matrix.}
}

\author{
  Hao Wu<hao.wu@emory.edu>
}

\seealso{
  DMLtest.multiFactor, DMLtest
}

\examples{
\dontrun{
data(RRBS)
## model fitting
DMLfit = DMLfit.multiFactor(RRBS, design, ~case+cell+case:cell)

## with smoothing:
DMLfit.sm = DMLfit.multiFactor(RRBS, design, ~case+cell+case:cell, smoothing=TRUE)

## hypothesis testing
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef=3)

## look at distributions of test statistics and p-values
par(mfrow=c(1,2))
hist(DMLtest.cell$stat, 100, main="test statistics")
hist(DMLtest.cell$pvals, 100, main="P values")
}
}

\name{callDMR}
\alias{callDMR}

\title{
  Function to detect differntially methylated regions (DMR) from
  bisulfite sequencing (BS-seq) data. 
}
\description{
  This function takes the results from DML testing procedure ('callDML'
  function) and calls DMRs. Regions will CpG sites being statistically
  significant are detected as DMRs. Nearby DMRs are merged into longer
  ones. Some restrictions including the minimum length, minimum number
  of CpG sites, etc. are applied. 
}
\usage{
callDMR(DMLresult, delta=0, p.threshold=1e-5,
        minlen=50, minCG=3, dis.merge=100, pct.sig=0.5)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{DMLresult}{
    A data frame representing the results for DML detection. This should
    be the result returned from 'DMLtest' or 'DMLtest.multiFactor' function.
  }
  \item{delta}{A threshold for defining DMR. In DML detection procedure, a hypothesis
    test that the two groups means are equal is conducted at each CpG
    site. Here if 'delta' is specified, the function will compute the
    posterior probability that the difference of the means are greater
    than delta, and then construct DMR based on that. This only works
	when the test results are from 'DMLtest', which is for two-group
	comparison.}
  \item{p.threshold}{
    A threshold of p-values for calling DMR. Loci with p-values less
    than this threshold will be picked and joint to form the DMRs. See
    'details' for more information.
  }
  \item{minlen}{
    Minimum length (in basepairs) required for DMR. Default is 50 bps. 
  }
  \item{minCG}{
    Minimum number of CpG sites required for DMR. Default is 3. 
  }
  \item{dis.merge}{
    When two DMRs are very close to each other and the distance (in bps)
    is less than this number, they will be merged into one. Default is
    50 bps. 
  }
  \item{pct.sig}{
    In all DMRs, the percentage of CG sites with significant p-values
    (less than p.threshold) must be greater than this threshold. Default
    is 0.5. This is mainly used for correcting the effects of merging of nearby
    DMRs. 
  }
}

\value{A data frame for DMRs. Each row is for a DMR. Rows are sorted by
  "areaStat", which is the sum of test statistics of all CpG sites in
  the region. The columns are:
  \item{chr}{Chromosome number.}
  \item{start, end}{Genomic coordinates.}
  \item{length}{Length of the DMR, in bps.}
  \item{nCG}{Number of CpG sites contained in the DMR.}
  \item{meanMethy1, meanMethy2}{Average methylation levels in two
    conditions.}
  \item{diff.Methy}{The difference in the methylation levels between two
    conditions. diff.Methy=meanMethy1-meanMethy2.}
  \item{areaStat}{The sum of the test statistics of all CpG sites within
    the DMR.}
}

\details{
  The choices of 'delta' and 'p.threshold' are somewhat arbitrary.
  The default value for p-value threshold for calling DMR is 1e-5. The
  statistical test on loci level is less powerful when smoothing is NOT
  applied, so users can consider to use a less stringent criteria, such
  as 0.001, in order to get satisfactory number of DMRs.
  This function is reasonably fast since the computationally
  intesnsive part is in 'DMLtest'. Users can try different 
  p.threshold values to obtain satisfactory results.

  'delta' is only supported when the experiment is for two-group
  comparison. This is because in multifactor design, the estimated
  coefficients in the regression are based on a GLM framework (loosely
  speaking), thus they don't have clear meaning of methylation level
  differences. So when the input DMLresult is from DMLtest.multiFactor,
  'delta' cannot be specified. 

  When specifying a 'delta' value, the posterior probability (pp) of
  each CpG site being DML is computed. Then the p.threshold is applied on
  1-pp, e.g., sites with 1-pp<p.threshold is deemed significant. In this
  case, the criteria for DMR calling is more stringent and users might
  consider to use a more liberal p.threshold in order to get more
  regions.
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\seealso{
DMLtest, callDML
}

\examples{
\dontrun{
require(bsseq)
require(bsseqData)
data(BS.cancer.ex)

## take a small portion of data and test
BSobj <- BS.cancer.ex[140000:150000,]
dmlTest <- DMLtest(BSobj, group1=c("C1", "C2", "C3"), group2=c("N1","N2","N3"),
   smoothing=TRUE, smoothing.span=500)

## call DMR based on test results
dmrs <- callDMR(dmlTest)
head(dmrs)

## or one can specify a threshold for difference in methylation level
dmrs2 <- callDMR(dmlTest, delta=0.1)
head(dmrs2)

## visualize one DMR
showOneDMR(dmrs[1,], BSobj)

## from multifactor design - using a loose threshold to demonstrate

data(RRBS)
DMLfit = DMLfit.multiFactor(RRBS, design, ~case+cell+case:cell)
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef="cellrN")
dmr = callDMR(DMLtest.cell, p.threshold=0.05) 
dmr
}
}

\name{seqData}
\alias{seqData}
\docType{data}
\title{
  A simulated 'SeqCountData' object.
}
\description{
  The object is created based on simulation for 1000 genes and two
  treatment groups with 4 replicates in each group.
}
\usage{data(seqData)}

\examples{
data(seqData)
seqData
}
\keyword{datasets}

\name{DMLtest}
\alias{DMLtest}
\title{
  Function to perform statistical test of differntially methylated loci
  (DML) for two group comparisons of bisulfite sequencing (BS-seq) data.
}

\description{
  This function takes a BSseq object and two group labels, then perform
  statistical tests for differntial methylation at each CpG site.
}

\usage{
DMLtest(BSobj, group1, group2, equal.disp = FALSE, smoothing = FALSE,
        smoothing.span = 500, ncores) 
}

\arguments{
  \item{BSobj}{An object of BSseq class for the BS-seq data.}
  \item{group1, group2}{Vectors of sample names or indexes for the two
    groups to be tested. See more description in details.}
  \item{equal.disp}{A flag to indicate whether the dispersion in two
    groups are deemed equal. Default is FALSE, and the dispersion
    shrinkages are performed on two conditions independently.}
  \item{smoothing}{A flag to indicate whether to apply smoothing in
    estimating mean methylation levels.}
%   \item{smoothing.method}{Method for smoothing. Available options are
%     "ma" for moving average, or "BSmooth" for BSmooth smoothing
%     method. This will be ignored if smoothing=FALSE.}
  \item{smoothing.span}{The size of smoothing window, in
    basepairs. Default is 500.}
  \item{ncores}{Number of CPU cores used in parallel computing.
	See sections 'Parallelization' for details.}
}

\details{
  This is the core function for DML/DMR detection. Tests are performed
  at each CpG site under the null hypothesis that two groups means are
  equal. There is an option for applying smoothing or not in estimating
  mean methylation levels. We recommend to use smoothing=TRUE for
  whole-genome BS-seq data, and smoothing=FALSE for sparser data such
  like from RRBS or hydroxyl-methylation data (TAB-seq). If there is not
  biological replicate, smoothing=TRUE is required. See "Single
  replicate" section for details.  
  
  The BS-seq count data are modeled as Beta-Binomial distribution, where
  the biological variations are captured by the dispersion
  parameter. The dispersion parameters are estimated through a shrinakge
  estimator based on a Bayesian hierarchical model. Then a Wald test is
  performed at each CpG site. 

  Due to the differences in coverages, some CpG sites are not covered in
  both groups, and the test cannot be performed. Those loci will be
  ignored in test and results will be "NA". 
}

\section{Single replicate}{
  When there is no biological replicate in one or both treatment groups,
  users can either (1) specify equal.disp=TRUE, which assumes both groups
  have the same dispersion, then the data from two groups are combined and
  used as replicates to estimate dispersion; or (2) specify
  smoothing=TRUE, which uses the smoothed means (methylation levels)
  to estimate dispersions via a shrinkage estimator. This smoothing
  procedure uses data from neighboring CpG sites as "pseudo-replicate"
  for estimating biological variance.}

\section{Parallelization}{
  The shrinkage estimation for dispersion is the most computational
  component in DML testing. We use the 'mcapply' function in 'parallel'
  package to implement parallelization. Note that older version of DSS
  (<2.4x) used the 'bplapply' function in 'BiocParallel'
  package. However, that function somehow has significantly reduced
  performance in the new release (>1.25), so we switched to mcapply.
  A drawback is that the progress bar cannot be displayed under the
  paralelle computing setting. 

  Users might experience problems on Windows, since the mcapply function
  relies on forking but Windows does not support forking. Thus, we
  suggest to use ncores=1 on Windows. For more details, please read the
  'parallel' documentation.}

  % All parallelization options are controlled via the 'BPPARAM'
  % argument. BiocParallel provides params for 
  % SerialParam (Unix, Mac, Windows),
  % MulticoreParam (Unix and Mac), SnowParam (Unix, Mac, and
  % Windows, limited to single-machine clusters), and
  % BatchJobsParam (Unix, Mac, Windows). By default, 'BPPARAM' takes value
  % from 'bpparam()'. Example section provides some codes to generate
  % user-specified BPPARAM. For more details, please  read the
  % 'BiocParallel' documentation.}
  
  

\value{A data frame with each row corresponding to a CpG site. Rows are
  sorted by chromosome number and genomic coordinates. The columns
  include:
  \item{chr}{Chromosome number.}
  \item{pos}{Genomic coordinates.}
  \item{mu1, mu2}{Mean methylations of two groups.}
  \item{diff}{Difference of mean methylations of two
  groups. diff=mu1-mu2. }
  \item{diff.se}{Standard error of the methylation difference.}
  \item{stat}{Wald statistics.}
  \item{pval}{P-values. This is obtained from normal distribution.}
  \item{fdr}{False discovery rate.}
}

\section{Estimating mean methylation levels}{
  When smoothing=FALSE, the mean methylation levels are estimated based
  on the ratios of methylated and total read counts, and the spatial
  correlations among nearby CpG sites are ignored. When smoothing=TRUE,
  smoothing based on moving average or the BSmooth method is used to
  estimate the mean methylaion level at each site. Moving average is
  recommended because it is much faster than BSmooth, and the results
  are reasonable similar in terms of mean estimation, dispersion
  estimation, and DMR calling results.
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\seealso{
makeBSseqData, callDML, callDMR
}

\examples{
\dontrun{
require(bsseq)

## first read in methylation data.
path <- file.path(system.file(package="DSS"), "extdata")
dat1.1 <- read.table(file.path(path, "cond1_1.txt"), header=TRUE)
dat1.2 <- read.table(file.path(path, "cond1_2.txt"), header=TRUE)
dat2.1 <- read.table(file.path(path, "cond2_1.txt"), header=TRUE)
dat2.2 <- read.table(file.path(path, "cond2_2.txt"), header=TRUE)

## make BSseq objects
BSobj <- makeBSseqData( list(dat1.1, dat1.2, dat2.1, dat2.2),
  c("C1","C2", "N1", "N2") )

##  DML test without smoothing 
dmlTest <- DMLtest(BSobj, group1=c("C1", "C2"), group2=c("N1", "N2"))
head(dmlTest)

## For whole-genome BS-seq data, perform DML test with smoothing
require(bsseqData)
data(BS.cancer.ex)
## take a small portion of data and test
BSobj <- BS.cancer.ex[10000:15000,]
dmlTest <- DMLtest(BSobj, group1=c("C1", "C2", "C3"), group2=c("N1","N2","N3"), 
                   smoothing=TRUE, smoothing.span=500)
head(dmlTest)

## Examples for Parallelization
## use single core - this has not parallelization
system.time(dmlTest <- DMLtest(BSobj, group1=c("C1", "C2"), group2=c("N1", "N2"), ncores=1))

## use 4 cores - it's about twice as fast
system.time(dmlTest <- DMLtest(BSobj, group1=c("C1", "C2"), group2=c("N1", "N2"), ncores=4))

% ## use SnowParam, this will work on Windows
% snow <- SnowParam(workers = numCores)
% dmlTest <- DMLtest(BSobj, group1=c("C1", "C2"), group2=c("N1", "N2"),
%                    BPPARAM=snow)
}
}

\name{estDispersion}
\alias{estDispersion,SeqCountSet-method}
\alias{estDispersion}

\title{
  Estimate and shrink tag-specific dipsersions 
}
\description{
  This function first estimate tag-specific dipsersions using a method
  of moment estimator. Then the dipsersions are shrunk based a penalized
  likelihood approach. The function works for general experimental
  designs. 
}

\usage{
\S4method{estDispersion}{SeqCountSet}(seqData, trend=FALSE)
}
\arguments{
  \item{seqData}{An object of SeqCountSet class.}
  \item{trend}{A binary indicator for modeling the dispersion~expression
  trend.} 
}
\details{
  The function takes and object of seqCountData class and return the
  same oject with ``dispersion'' field filled.

  With ``trend=TRUE'' the dependence of dispersion on mean expressions
  will be modeled. In that case the shrinkage will be performed
  conditional on mean expressions.

  The function works for multiple factor designs. But option
  ``trend=TRUE'' only applicable for single factor experiment. 
}

\examples{
data(seqData)
seqData=estNormFactors(seqData)
seqData=estDispersion(seqData)
head(dispersion(seqData))

## For multiple factor design
data(seqData)
Y=exprs(seqData)
design=data.frame(gender=c(rep("M",4), rep("F",4)), strain=rep(c("WT", "Mutant"),4))
X=as.data.frame(model.matrix(~gender+strain, data=design))
seqData=newSeqCountSet(Y, X)
seqData=estDispersion(seqData)
head(dispersion(seqData))

## the hypothesis testing for multifactor experiments can be performed
## using edgeR function, with DSS estimated dispersions
\dontrun{
library(edgeR)
fit.edgeR <- glmFit(Y, X, lib.size=normalizationFactor(seqData), dispersion=dispersion(seqData))
lrt.edgeR <- glmLRT(fit.edgeR, coef=2)
head(lrt.edgeR$table)
}
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\keyword{RNA-seq}

\name{DMLtest.multiFactor}
\alias{DMLtest.multiFactor}

\title{
  Perform statistical test for BS-seq data from general experimental design
}

\description{
  This function takes the linar model fitting results and performs Wald
  test at each CpG site, then return test statistics, p-values and FDR.
}

\usage{
DMLtest.multiFactor(DMLfit, coef = 2, term, Contrast)
}

\arguments{
  \item{DMLfit}{Result object returned from 'DMLfit.multiFactor' function.}
  \item{coef}{It can be an integer to indicate which coefficient in the linear model
	is be tested for being zero. Be careful of intercept. If the model
	contains intercept, coef=2 indicate testing the first factor in the
	formula. If the model has no intercept, testing first factor should
	use coef=1.

	It can also be a character for the terms to be tested. In that case it
	must match one of the column names in the design matrix. One can
	look at colnames(DMLfit$X) to obtain the column names. 
  }
  \item{term}{The term(s) to be tested, as one or a vector of
	characters. Can be multiple terms. See 'Hypothesis test' section for
	details.} 
  \item{Contrast}{A contrast matrix for hypothesis testing. The number
	of rows must equal to the number of columns in the design matrix
	ncol(DMLfit$X). See 'Hypothesis test' section for details.}																
}

\section{Hypothesis test}{
  User can specify one of the following parameter for testing:
  'coef', 'term', or 'Contrast'.

  When specifying 'coef', it tests *one* parameter in the model, which
  corresponds to one column in the design matrix. In this case,   
  A Wald test is performed using the estimated coefficient and standard
  error from 'DMLfit.multiFactor'. P-values are derived from test
  statistics based on normal distribution.

  When specifying 'term', it tests the whole term in the model. If the
  term is continuous or a categorical variable with only two levels (one
  degree of freedom), it is equivalent to specifying 'coef' because it
  tests only one parameter in the model. However, when the term is a
  categorical variable with more than two levels, it will test multiple
  parameters at the same time so it's a compound hypothesis test, and a
  F-test will be performed. 
  
  Specifying 'Contrast' matrix provides the most flexible test
  procedure. It can test any linear combination of the parameter, and
  F-test will be performed. Let L be the contrast matrix. The hypothesis
  test performed is H0: L^T * beta = 0. Thus the number of rows in L must
  equal to the number of items of beta (which is the number of columns
  in the design matrix). 

  Using 'term' or 'Contrast' will be slower especially when there are a
  lot of CpG sites, because the computation cannot be vectorized (each
  CpG site has a different variance/covariance matrix for the estimated
  coefficients).
  
  FDR is computed using cannonical Benjamini-Hochberg procedure.
}

\value{
  A data frame with following columns: chr, pos, stat, pvals, fdr. Each
  row is for a CpG site. Note that the CpG sites are sorted by
  chromosome and position. 
}

\author{
  Hao Wu<hao.wu@emory.edu>
}

\seealso{
  DMLfit.multiFactor, DMLtest
}

\examples{
\dontrun{
data(RRBS)
## model fitting
DMLfit = DMLfit.multiFactor(RRBS, design, ~case+cell+case:cell)

## hypothesis testing - following two lines do the same thing
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef=3)
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef="cellrN")

## this doesn't work
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef="cell")

## look at distributions of test statistics and p-values
par(mfrow=c(1,2))
hist(DMLtest.cell$stat, 100, main="test statistics")
hist(DMLtest.cell$pvals, 100, main="P values")

## Using term or Contrast
DMLfit = DMLfit.multiFactor(RRBS, design, ~case+cell)

## following 4 tests should produce the same results,
## since 'case' only has two levels.
## However the p-values from F-tests (using term or Contrast) are
## slightly different, due to normal approximation in Wald test.
test1 = DMLtest.multiFactor(DMLfit, coef=2)
test2 = DMLtest.multiFactor(DMLfit, coef="caseSLE")
test3 = DMLtest.multiFactor(DMLfit, term="case")
Contrast = matrix(c(0,1,0), ncol=1)
test4 = DMLtest.multiFactor(DMLfit, Contrast=Contrast)
cor(cbind(test1$pval, test2$pval, test3$pval, test4$pval))


## note the different usage of term and coef.
## 'term' has to be in the formula, whereas 'coef' has to be in colnames
## of the design matrix.
DMLfit = DMLfit.multiFactor(RRBS, design, ~case+cell+case:cell)
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef="cellrN")
DMLtest.cell = DMLtest.multiFactor(DMLfit, term="cell")
DMLtest.int = DMLtest.multiFactor(DMLfit, coef="caseSLE:cellrN")
DMLtest.int = DMLtest.multiFactor(DMLfit, term="case:cell")

}
}

\name{showOneDMR}
\alias{showOneDMR}
\title{
  Visualze the count data for one DMR
}
\description{
  Given one DMR and an BSseq object, this function generate a multiple
  panel figure, each for a sample, to visualze the counts. There is a
  bar at each CpG, the gray bar shows the total coverage, and the black
  bar shows the methylated count. 
}
\usage{
showOneDMR(OneDMR, BSobj, ext = 500, ylim)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{OneDMR}{A data frame with one row representing one DMR. It must
    have chr, start, and end columns. This is typically a row from the
    result generated from callDMR.}
  \item{BSobj}{An object of class BSseq.}
  \item{ext}{The amount (in bps) the plotting region should be extended
    in both directions.}
  \item{ylim}{Y-axis limit.}
}

\value{
  This function only generates a figure and has no return values.
}

\author{
Hao Wu <hao.wu@emory.edu>
}

\seealso{
  callDMR
}
\examples{
\dontrun{
require(bsseq)
require(bsseqData)
data(BS.cancer.ex)

## takea small portion of data and test
BSobj <- BS.cancer.ex[140000:150000,]
dmlTest <- DMLtest(BSobj, group1=c("C1", "C2", "C3"), group2=c("N1","N2","N3"),
   smoothing=TRUE, smoothing.span=500)

## call DMR based on test results
dmrs <- callDMR(dmlTest)

## visualize one DMR
showOneDMR(dmrs[1,], BSobj)
}

}

\name{dispersion}
\alias{dispersion,SeqCountSet-method}
\alias{dispersion<-,SeqCountSet,numeric-method}
\alias{dispersion}
\alias{dispersion<-}
\title{
  Acessor functions for the 'dispersion' slot in a
  SeqCountData object.
}
\description{
  Dispersion parameter for a gene represents its coefficient of
  variation of expressions. It characterizes the biological
  variations. 
}
\usage{
\S4method{dispersion}{SeqCountSet}(object)
\S4method{dispersion}{SeqCountSet,numeric}(object) <- value
}
\arguments{
  \item{object}{A SeqCountData object.}
  \item{value}{A numeric vector with the same length as number of genes.}
}

\details{
  If the counts from biological replicates are modeled as negative
  binomial distribution, the variance (v) and mean (m) should hold
  following relationship: v=m+m^2*phi, where phi is the
  dispersion. Another interpretation is that phi represents the
  biological variations among replicates when underlying expressions are
  modeled as a Gamma distribution.
}
\author{
  Hao Wu <hao.wu@emory.edu>
}
\seealso{
  normalizationFactor
}
\examples{
data(seqData)
## obtain
seqData=estNormFactors(seqData, "quantile")
seqData=estDispersion(seqData)
dispersion(seqData)

## assign
dispersion(seqData)=rep(0.1, nrow(exprs(seqData)))

}

\name{RRBS}
\alias{RRBS}
\docType{data}
\title{An example dataset for multiple factor design}
\description{
  The dataset contains RRBS data for 5000 CpG sites from 16 samples. The
  experimental design is provided in the 'design' data frame.
}
\usage{data("RRBS")}
\examples{
data(RRBS)
RRBS
design
}
\keyword{datasets}

\name{waldTest}
\alias{waldTest,SeqCountSet-method}
\alias{waldTest}
\title{
  Perform gene-wise Wald test for two group comparisons for sequencing
  count data. 
}
\description{
  The counts from two groups are modeled as negative binomial random
  variables with means and dispersions estimated. Wald statistics will
  be constructed. P-values will be obtained based on Gaussian assumption.
}
\usage{
\S4method{waldTest}{SeqCountSet}(seqData, sampleA, sampleB, equal.var, fdr.method=c("BH", "locfdr"))
}

\arguments{
  \item{seqData}{An object of SeqCountSet class.}
  \item{sampleA}{The sample labels for the first sample to be
    compared in two-group comparison.}
  \item{sampleB}{The sample labels for the second sample to be
    compared in two-group comparison.}
  \item{equal.var}{A boolean to indicate whether to use the same or
    different means in two groups for computing variances in Wald
    test. Default is FALSE.}
    \item{fdr.method}{Method to compute FDR. Availabile options are "BH" for 
    Benjamini-Hochberg FDR method, or local FDR from "locfdr" package.}
}

\details{
  The input seqCountData object Must have normalizationFactor and
  dispersion fields filled, e.g., estNormFactors and estDispersion need
  to be called prior to this. With group means and shrunk dispersions
  ready, the variances for difference in group means will be constructed
  based on Negative Binomial distribution. P-values will be obtained
  under the assumption that the Wald test statistics are normally
  distributed. Genes with 0 counts in both groups will be assigned 0 for
  test statistics and 1 for p-values.
}

\value{
  A data frame with each row corresponding to a gene. Rows are sorted
  according to wald test statistics. The columns are:
  \item{gene Index}{index for input gene orders, integers from 1 to the
    number of genes.}
  \item{muA}{sample mean (after normalization) for sample A.}
  \item{muB}{sample mean (after normalization) for sample B.}
  \item{lfc}{log fold change of expressions between two groups.}
  \item{difExpr}{differences in expressions between two groups.}
  \item{stats}{Wald test statistics.}
  \item{pval}{p-values.}
  \item{fdr}{FDR.}
  \item{local.fdr}{Local FDR if the FDR method is "locfdr".}
  \item{others}{input gene annotations supplied as AnnotatedDataFrame
    when constructed the SeqCountData object.}
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\examples{
data(seqData)
seqData=estNormFactors(seqData)
seqData=estDispersion(seqData)
result=waldTest(seqData, 0, 1)
head(result)
}

\name{estNormFactors}
\alias{estNormFactors,SeqCountSet-method}
\alias{estNormFactors}

\title{
  Estimate normalization factors
}
\description{
  This function estimates normalization factors for the input
  'seqCountSet' object and return the same object with
  normalizationFactor field filled or replaced.
}
\usage{
\S4method{estNormFactors}{SeqCountSet}(seqData, method=c("lr", "quantile", "total", "median"))
}

\arguments{
  \item{seqData}{An object of "SeqCountSet" class. }
  \item{method}{Methods to be used in computing normalization
    factors. Currently available options only include methods to compute
    normalization factor to adjust for sequencing depths.
    Available options use (1) "lr": using median of logratio of
	counts. Similar to the TMM method. (2) "quantile"
    (default):  75th quantile, (3) "total":  total counts, or (4)
    "median":  median counts to constuct the normalization
    factors. From all methods the normalization factor will be a vector
    with same length as number of columns for input counts.}
}

\value{
  The same "SeqCountSet" object with normalizationFactor field filled or
  replaced. 
}

\examples{
data(seqData)
## compare different methods
seqData=estNormFactors(seqData, "lr")
k1=normalizationFactor(seqData)
seqData=estNormFactors(seqData, "quantile")
k2=normalizationFactor(seqData)
seqData=estNormFactors(seqData, "total")
k3=normalizationFactor(seqData)
cor(cbind(k1,k2,k3))

## assign size factor
normalizationFactor(seqData)=k1

## or normalization factor can be a matrix
dd=exprs(seqData)
f=matrix(runif(length(dd), 1,10), nrow=nrow(dd), ncol=ncol(dd))
normalizationFactor(seqData)=f
head(normalizationFactor(seqData))
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\keyword{normalization}

\name{normalizationFactor}
\alias{normalizationFactor,SeqCountSet-method}
\alias{normalizationFactor<-,SeqCountSet,numeric-method}
\alias{normalizationFactor<-,SeqCountSet,matrix-method}
\alias{normalizationFactor}
\alias{normalizationFactor<-}

\title{
  Accessor functions for the 'normalizationFactor' slot in a
  SeqCountData object.
  
}
\description{
  The normalization factors are used to adjust for technical or
  biological biases in the sequencing experiments. The factors 
  can either be (1) a vector with length equals
  to the number of columns of the count data; or (2) a matrix with the same
  dimension of the count data.
}
\usage{
\S4method{normalizationFactor}{SeqCountSet}(object)
\S4method{normalizationFactor}{SeqCountSet,numeric}(object) <- value
\S4method{normalizationFactor}{SeqCountSet,matrix}(object) <- value
}

\arguments{
  \item{object}{A SeqCountData object.}
  \item{value}{A numeric vector or matrix. If it is a vector it must
    have length equals to the number of columns of the count data. For
    matrix it must have the same dimension of the count data.}
}

\details{
  The vector normalization factors are used mostly to
  correct for sequencing depth from different datasets. The matrix
  factor applies a different normalizing constant for each gene at each
  sample to adjust for a broader range of artifacts such as GC content. 
}
  
\author{
  Hao Wu <hao.wu@emory.edu>
}

\seealso{
  dispersion
}
\examples{
data(seqData)
## obtain nomalization factor
seqData=estNormFactors(seqData, "quantile")
normalizationFactor(seqData)

## assign as vector
normalizationFactor(seqData)=rep(1, ncol(exprs(seqData))) ## getan error here

## or assign as a matrix
f=matrix(1, nrow=nrow(exprs(seqData)), ncol=ncol(exprs(seqData)))
normalizationFactor(seqData)=f
}

\name{SeqCountSet-class}
\Rdversion{1.1}
\docType{class}
\alias{SeqCountSet-class}
\alias{SeqCountSet}

% Constructor-like function:
\alias{newSeqCountSet}

\title{Class \code{"SeqCountSet"} - container for count data from
  sequencing experiment}

\description{
  This class is the main container for storing *RNA-seq* data.
  It is directly inherited fro 'ExpressionSet' class, with two
  more fields 'normalizationFactor' for normalization factors and
  'dispersion' for gene-wise dispersions.

  The class for BS-seq data is *BSseq*, which is imported from bsseq
  package. 
}

\section{Slots}{
    \describe{
      \item{\code{normalizationFactor}:}{Normalization factor for counts.}
      \item{\code{dispersion}:}{Gene-wise dispersions.}
      \item{\code{experimentData}:}{See 'ExpressionSet'.}
      \item{\code{assayData}:}{See 'ExpressionSet'.}
      \item{\code{phenoData}:}{See 'ExpressionSet'.}
      \item{\code{featureData}:}{See 'ExpressionSet'.}
      \item{\code{annotation}:}{See 'ExpressionSet'.}
      \item{\code{protocolData}:}{See 'ExpressionSet'.}
  }
}
  
\section{Extends}{
  Class \code{"\linkS4class{ExpressionSet}"}, directly.
  Class \code{"\linkS4class{eSet}"}, by class "ExpressionSet", distance 2.
  Class \code{"\linkS4class{VersionedBiobase}"}, by class "ExpressionSet", distance 3.
  Class \code{"\linkS4class{Versioned}"}, by class "ExpressionSet", distance 4.
}

\section{Constructor}{
  \describe{
    \item{}{
      \code{newSeqCountSet(counts,
	designs,normalizationFactor,featureData)}:
      Creates a 'SeqCountSet' object.
      \describe{
	\item{\code{counts}}{A matrix of integers with rows corresponding to
	  genes and columns for samples.}
	\item{\code{designs}}{A vector or data frame representing experimental
	  design. The length of the vector or number of rows of the
	  data frame must match the number of columns of
	  input counts. This field can be accessed using 'pData' function.}
	\item{\code{normalizationFactor}}{A vector or matrix of
	  normalization	factors for the counts. }
	\item{\code{featureData}}{Additional information for genes
	  as an 'AnnotatedDataFrame' object. This field can be
	  access by using 'featureData' function.}
      }
    }
  }
}


\section{Methods}{
  \describe{
    \item{dispersion, dispersion<- :}{Access and set gene-wise dispersions.}
    \item{normalizationFactor, normalizationFactor<- :}{Access and set
      normalization factors.}
  }
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\seealso{
  dispersion, normalizationFactor
}

\note{
  This is similar to 'CountDataSet' in DESeq or 'DGEList' in edgeR. 
}

\examples{
## simulate data from RNA-seq
counts=matrix(rpois(600, 10), ncol=6)
designs=c(0,0,0,1,1,1)
seqData=newSeqCountSet(counts, designs)
seqData
pData(seqData)
head(exprs(seqData))

## multiple factor designs
design=data.frame(gender=c(rep("M",4), rep("F",4)), strain=rep(c("WT", "Mutant"),4))
X=model.matrix(~gender+strain, data=design)
counts=matrix(rpois(800, 10), ncol=8)
seqData=newSeqCountSet(counts, as.data.frame(X))
seqData
pData(seqData)

}
\keyword{classes}

\name{makeBSseqData}
\alias{makeBSseqData}

\title{
  Create an object of BSseq class from several data frames.
}
\description{
  This is an utility function to merge BS-seq data from replicated
  experiment and create an object of BSseq class.
  
  After sequence alignment and proper processing, the BS-seq data can be
  summarized by following information at each C position (mostly CpG
  sites, with some CH): chromosome number, genomic coordinate, total
  number of reads covering the position, and number of reads showing
  methylation at this position. For replicated samples, the data need to
  be merged based on the chromosome number and genomic coordinates. This
  function provide such functionality. It takes replicated data as a
  list of data frames, merged them, and create a BSseq object.
}
\usage{
makeBSseqData(dat, sampleNames)
}

\arguments{
  \item{dat}{A list of multiple data frames from biological
    replicates. Each element represents data from one replicate. The data
    frame MUST contain following columns in correct order: (1) Chromosome number; (2)
    Genomic coordinates; (3) Read coverage of the position from BS-seq
    data; (4) Number of reads showing methylation of the position. The
	colnames MUST BE "chr", "pos", "N", "X".}
  \item{sampleNames}{A vector of characters for the sample names. The
    length of the vector should match the length of the input list.}
}

\value{
  An object of 'BSseq' class.
}

\author{
Hao Wu <hao.wu@emory.edu>
}

\seealso{
callDML
}

\examples{
require(bsseq)

## first read in methylation data.
path <- file.path(system.file(package="DSS"), "extdata")
dat1.1 <- read.table(file.path(path, "cond1_1.txt"), header=TRUE)
dat1.2 <- read.table(file.path(path, "cond1_2.txt"), header=TRUE)
dat2.1 <- read.table(file.path(path, "cond2_1.txt"), header=TRUE)
dat2.2 <- read.table(file.path(path, "cond2_2.txt"), header=TRUE)

## make BSseq objects
BSobj <- makeBSseqData( list(dat1.1, dat1.2, dat2.1, dat2.2),
  c("C1","C2", "N1", "N2") )

BSobj
sampleNames(BSobj)
}

\name{callDML}
\alias{callDML}

\title{
  Function to detect differntially methylated loci (DML) from bisulfite
  sequencing (BS-seq) data. 
}
\description{
  This function takes the results from DML testing procedure ('DMLtest'
  function) and calls DMLs. Regions will CpG sites being statistically
  significant are deemed as DMLs. 
}
\usage{
callDML(DMLresult, delta=0.1, p.threshold=1e-5)
}

\arguments{
  \item{DMLresult}{
    A data frame representing the results for DML detection. This should
    be the result returned from 'DMLtest' function.}
  \item{delta}{A threshold for defining DML. In DML testing procedure, 
    hypothesis test that the two groups means are equalis is conducted at each CpG
    site. Here if 'delta' is specified, the function will compute
    the posterior probability that the difference of the means are
    greater than delta,and then call DML based on that. This only works
    when the test results are from 'DMLtest', which is for two-group
    comparison. For general design, this has to be set to 0.}
  \item{p.threshold}{
    When delta is not specified, this is the threshold of p-values for
	defining DML, e.g. Loci with p-values less than this threshold will
	be deemed DMLs. When delta is specified, CpG sites with posterior
	probability greater than 1-p.threshold are deemed DML.
  }
}

\value{A data frame for DMLs. Each row is for a DML. DMLs are sorted by
  statistical significance. The columns are
  \item{chr}{Chromosome number.}
  \item{pos}{Genomic coordinates.}
  \item{mu1, mu2}{Mean methylations of two groups.}
  \item{diff}{Difference of mean methylations of two groups.}
  \item{diff.se}{Standard error of the methylation difference.}
  \item{stat}{Wald statistics.}
  \item{phi1, phi2}{Estimated dispersions in two groups.}
  \item{pval}{P-values. This is obtained from normal distribution.}
  \item{fdr}{False discovery rate.}
  \item{postprob.overThreshold}{The posterior probability of the
	difference in methylation greater than delta. This columns is only
	available when delta>0.}
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\seealso{
DMLtest, callDMR
}

\examples{
\dontrun{
require(bsseq)

## first read in methylation data.
path <- file.path(system.file(package="DSS"), "extdata")
dat1.1 <- read.table(file.path(path, "cond1_1.txt"), header=TRUE)
dat1.2 <- read.table(file.path(path, "cond1_2.txt"), header=TRUE)
dat2.1 <- read.table(file.path(path, "cond2_1.txt"), header=TRUE)
dat2.2 <- read.table(file.path(path, "cond2_2.txt"), header=TRUE)

## make BSseq objects
BSobj <- makeBSseqData( list(dat1.1, dat1.2, dat2.1, dat2.2),
  c("C1","C2", "N1", "N2") )

##  DML test
dmlTest <- DMLtest(BSobj, group1=c("C1", "C2"), group2=c("N1","N2"))

## call DML
dmls <- callDML(dmlTest)
head(dmls)

## call DML with a threshold
dmls2 <- callDML(dmlTest, delta=0.1)
head(dmls2)

## For whole-genome BS-seq data, perform DML test with smoothing
require(bsseqData)
data(BS.cancer.ex)
## takea smallportionof data and test
BSobj <- BS.cancer.ex[10000:15000,]
dmlTest <- DMLtest(BSobj, group1=c("C1", "C2", "C3"), group2=c("N1","N2","N3"),
   smoothing=TRUE, smoothing.span=500)
dmls <- callDML(dmlTest)
head(dmls)

## from multifactor design
data(RRBS)
DMLfit = DMLfit.multiFactor(RRBS, design, ~case+cell+case:cell)
DMLtest.cell = DMLtest.multiFactor(DMLfit, coef="cellrN")
dml = callDML(DMLtest.cell, p.threshold=0.05) ## this produce a warning
dml = callDML(DMLtest.cell, delta=0, p.threshold=0.05) ## no warning

head(dml)

}
}

\name{DSS.DE}
\alias{DSS.DE}
\title{
  Perform RNA-seq differential expression analysis in two-group comparison
}
\description{
  This is the top level wrapper function for RNA-seq differential expression
  analysis in a two-group comparison. Users only need to provide the
  count matrix and a vector for design, and obtain DE test results. 
}
\usage{
DSS.DE(counts, design)
}
\arguments{
  \item{counts}{A matrix of integers with rows corresponding to
    genes and columns for samples.}
  \item{design}{A vector representing the treatment groups. It must be
	a vector of 0 and 1. The length of the vector must match the number
	of columns of input count matrix.}
}


\value{
  A data frame with each row corresponding to a gene. Rows are sorted
  according to wald test statistics. The columns are:
  \item{gene Index}{index for input gene orders, integers from 1 to the
    number of genes.}
  \item{muA}{sample mean (after normalization) for sample A.}
  \item{muB}{sample mean (after normalization) for sample B.}
  \item{lfc}{log fold change of expressions between two groups.}
  \item{difExpr}{differences in expressions between two groups.}
  \item{stats}{Wald test statistics.}
  \item{pval}{p-values.}
  \item{others}{input gene annotations supplied as AnnotatedDataFrame
    when constructed the SeqCountData object.}
}

\author{
  Hao Wu <hao.wu@emory.edu>
}

\examples{
counts = matrix(rpois(600, 10), ncol=6)
design = c(0,0,0,1,1,1)
result = DSS.DE(counts, design)
head(result)
}
